package com.hisense.monitormanage.controller;

import com.hisense.monitormanage.dto.BuildingRecordsDto;
import com.hisense.monitormanage.dto.BuildingRecordsDtos;
import com.hisense.monitormanage.dto.EventDto;
import com.hisense.monitormanage.dto.RoadDataDto;
import com.hisense.monitormanage.entity.BuildingRecords;
import com.hisense.monitormanage.entity.Event;
import com.hisense.monitormanage.entity.Result;
import com.hisense.monitormanage.entity.RoadData;
import com.hisense.monitormanage.mapper.BuildingRecordsMapper;
import com.hisense.monitormanage.mapper.RoadDataMapper;
import com.hisense.monitormanage.service.BuildingRecordsService;
import com.hisense.monitormanage.utils.LongLatUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author admin
 * @version 1.0.0
 * @ClassName BuildingController.java
 * @Description 工地
 * @createTime 2022年05月17日 15:26:00
 */
@Api(tags = "工地")
@RequestMapping("api/project/building")
@RestController
public class BuildingController {

    @Autowired
    private BuildingRecordsService buildingRecordsService;

    @Autowired
    private BuildingRecordsMapper buildingRecordsMapper;

    @Autowired
    private RoadDataMapper roadDataMapper;

    @GetMapping("/getRecords")
    @ApiOperation("测试，获取工地实时数据--调用接口")
    public List<Map> getRecords() {
        return buildingRecordsService.getRecords();
    }

    @GetMapping("/getAndSaveRecords")
    @ApiOperation("获取工地实时数据并保存到表中--调用接口获取")
    public Result getAndSaveRecords() {
        boolean result = buildingRecordsService.getAndSaveRecords();
        if (result) {
            return Result.success();
        } else {
            return Result.error(String.valueOf(result));
        }
    }

    @GetMapping("/listRecords")
    @ApiOperation("获取工地实时数据列表-从t_building_records表中查询")
    @ApiImplicitParam(name = "pushTime", value = "上传时间", paramType = "query", required = true, dataType = "string", example = "2022-01-01")
    public List<BuildingRecords> listRecords(
            @RequestParam(value = "pushTime") String pushTime
    ) {
        return buildingRecordsService.listRecords(pushTime);
    }


    /**
     * 查询所有工地最新列表
     *
     * @return
     */
    @GetMapping("selectAllBuilding")
    @ApiOperation("查询所有工地")
    public Result selectAllBuilding() {
        List<BuildingRecordsDto> records = buildingRecordsMapper.selectAllBuilding();
        records.forEach(record -> {
            record.setProjectName(record.getGdmc());
        });
        Result success = Result.success(records);
        return success;
    }

    /**
     * 根据建筑编号查询近7天工地噪声和扬尘的信息
     *
     * @return
     */
    @GetMapping("selectWeekPmAndNoice")
    @ApiOperation("根据建筑编号查询近7天工地噪声和扬尘的信息")
    @ApiImplicitParam(name = "buildLicense", value = "建筑许可编号", required = true, dataType = "String")
    public Result selectWeekPmAndNoice(String buildLicense) {
        List<BuildingRecordsDtos> records = buildingRecordsMapper.selectWeekPmAndNoice(buildLicense);
        records.forEach(record -> {
            if (record.getPm10() == null && record.getNoice() == null) {
                record.setPm10(0);
                record.setNoice(0);
            }
        });
        Result success = Result.success(records);
        return success;
    }

    /**
     * 根据建筑编号查询近30天工地噪声和扬尘的信息
     *
     * @return
     */
    @GetMapping("selectMonthPmAndNoice")
    @ApiOperation("根据建筑编号查询近30天工地噪声和扬尘的信息")
    @ApiImplicitParam(name = "buildLicense", value = "建筑许可编号", required = true, dataType = "String")
    public Result selectMonthPmAndNoice(String buildLicense) {
        List<BuildingRecordsDtos> records = buildingRecordsMapper.selectMonthPmAndNoice(buildLicense);
        records.forEach(record -> {
            if (record.getPm10() == null && record.getNoice() == null) {
                record.setPm10(0);
                record.setNoice(0);
            }
        });
        Result success = Result.success(records);
        return success;
    }

    /**
     * 查询工地扬尘事件
     *
     * @return
     */
    @GetMapping("selectRaise")
    @ApiOperation("查询工地扬尘事件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "页码", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页码大小", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", required = false, dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", required = false, dataType = "String")
    })
    public Result selectRaise(Integer page, Integer pageSize, String startTime, String endTime) {
        List<BuildingRecordsDto> records = buildingRecordsMapper.selectRaise(page, pageSize, startTime, endTime);
        Integer integer = buildingRecordsMapper.selectCountRaise();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", integer);
        map.put("data", records);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询工地噪声事件
     *
     * @return
     */
    @GetMapping("selectNoiceEvent")
    @ApiOperation("查询工地噪声事件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "页码", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页码大小", required = true, dataType = "Integer")
    })
    public Result selectNoiceEvent(Integer page, Integer pageSize) {
        List<BuildingRecordsDto> records = buildingRecordsMapper.selectNoiceEvent(page, pageSize);
        Integer integer = buildingRecordsMapper.selectCountNoice();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", integer);
        map.put("data", records);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询工地视频感知事件
     *
     * @return
     */
    @GetMapping("selectByAllEvent")
    @ApiOperation("查询工地视频感知事件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "页码", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页码大小", required = true, dataType = "Integer")
    })
    public Result selectByAllEvent(Integer page, Integer pageSize) {
        List<EventDto> eventDtos = buildingRecordsMapper.selectByAllEvent(page, pageSize);
        eventDtos.forEach(eventDto -> {
            eventDto.setEventNewName(eventDto.getEventCnName());
        });
        Integer integer = buildingRecordsMapper.selectCountEvent();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", integer);
        map.put("data", eventDtos);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 根据工地名称搜索工地
     *
     * @param projectName
     * @return
     */
    @GetMapping("selectByProjectName")
    @ApiOperation("根据工地名称搜索工地")
    @ApiImplicitParam(name = "projectName", value = "工地名称", required = true, dataType = "String")
    public Result selectByProjectName(String projectName) {
        List<BuildingRecordsDto> records = buildingRecordsMapper.selectByProjectName(projectName);
        Result success = Result.success(records);
        return success;
    }

    /**
     * 根据经纬度查询半径内所有的工地
     *
     * @param jd
     * @param wd
     * @param radius
     * @return
     */
    @GetMapping("selectByJdWd")
    @ApiOperation("根据经纬度查询半径内所有的工地")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "jd", value = "经度", required = true, dataType = "Double"),
            @ApiImplicitParam(name = "wd", value = "纬度", required = true, dataType = "Double"),
            @ApiImplicitParam(name = "radius", value = "半径，米", required = true, dataType = "Integer"),}
    )
    public Result selectByJdWd(Double jd, Double wd, Integer radius) {
        double[] around = LongLatUtil.getAround(jd, wd, radius);
        List<BuildingRecordsDto> buildingRecordsDtos = buildingRecordsMapper.selectByJdWd(around[0], around[2], around[1], around[3]);
        buildingRecordsDtos.forEach(buildingRecordsDto -> {
            buildingRecordsDto.setProjectName(buildingRecordsDto.getGdmc());
        });
        Result success = Result.success(buildingRecordsDtos);
        return success;
    }

    /**
     * 查询当天工地视频感知事件
     *
     * @return
     */
    @GetMapping("selectByEvent")
    @ApiOperation("查询当天工地视频感知事件")
    public Result selectByEvent() {
        List<EventDto> events = buildingRecordsMapper.selectByEvent();
        events.forEach(eventDto -> {
            eventDto.setEventNewName(eventDto.getEventCnName());
        });
        Map<String, Object> map = new HashMap<>();
        map.put("sum", events.size());
        map.put("data", events);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询当天扬尘事件
     *
     * @return
     */
    @GetMapping("selectByDayRaise")
    @ApiOperation("查询当天扬尘事件")
    public Result selectByDayRaise() {
        List<BuildingRecordsDto> records = buildingRecordsMapper.selectByDayRaise();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", records.size());
        map.put("data", records);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询当天噪声事件
     *
     * @return
     */
    @GetMapping("selectByDayNoice")
    @ApiOperation("查询当天噪声事件")
    public Result selectByDayNoice() {
        List<BuildingRecordsDto> records = buildingRecordsMapper.selectByDayNoice();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", records.size());
        map.put("data", records);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询当天道路污染事件
     *
     * @return
     */
    @GetMapping("selectByDayRoadData")
    @ApiOperation("查询当天道路污染事件")
    public Result selectByDayRoadData() {
        RoadData byMaxId = roadDataMapper.getByMaxId();
        String dt = byMaxId.getDt();
        List<RoadData> roadData = buildingRecordsMapper.selectByDayRoadData(dt);
        Map<String, Object> map = new HashMap<>();
        map.put("sum", roadData.size());
        map.put("data", roadData);
        Result success = Result.success(map);
        return success;
    }

    @GetMapping("exportEvent")
    public String exportRaiseEvent(String startTime, String endTime) throws Exception {
        buildingRecordsService.testHSSF(startTime, endTime);
        return "导出成功";
    }

    /**
     * 查询近7天噪声事件
     *
     * @return
     */
    @GetMapping("selectByWeekNoice")
    @ApiOperation("查询近7天噪声事件")
    public Result selectByWeekNoice() {
        List<BuildingRecordsDto> buildingRecordsDtos = buildingRecordsMapper.selectByWeekNoice();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", buildingRecordsDtos.size());
        map.put("data", buildingRecordsDtos);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询近30天噪声事件
     *
     * @return
     */
    @GetMapping("selectByMonthNoice")
    @ApiOperation("查询近30天噪声事件")
    public Result selectByMonthNoice() {
        List<BuildingRecordsDto> buildingRecordsDtos = buildingRecordsMapper.selectByMonthNoice();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", buildingRecordsDtos.size());
        map.put("data", buildingRecordsDtos);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询近7天扬尘事件
     *
     * @return
     */
    @GetMapping("selectByWeekNaise")
    @ApiOperation("查询近7天扬尘事件")
    public Result selectByWeekNaise() {
        List<BuildingRecordsDto> buildingRecordsDtos = buildingRecordsMapper.selectByWeekNaise();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", buildingRecordsDtos.size());
        map.put("data", buildingRecordsDtos);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询近30天扬尘事件
     *
     * @return
     */
    @GetMapping("selectByMonthNaise")
    @ApiOperation("查询近30天扬尘事件")
    public Result selectByMonthNaise() {
        List<BuildingRecordsDto> buildingRecordsDtos = buildingRecordsMapper.selectByMonthNaise();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", buildingRecordsDtos.size());
        map.put("data", buildingRecordsDtos);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询近7天工地视频感知事件
     *
     * @return
     */
    @GetMapping("selectByWeekEvent")
    @ApiOperation("查询近7天工地视频感知事件")
    public Result selectByWeekEvent() {
        List<EventDto> events = buildingRecordsMapper.selectByWeekEvent();
        events.forEach(eventDto -> {
            eventDto.setEventNewName(eventDto.getEventCnName());
        });
        Map<String, Object> map = new HashMap<>();
        map.put("sum", events.size());
        map.put("data", events);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询近30天工地视频感知事件
     *
     * @return
     */
    @GetMapping("selectByMonthEvent")
    @ApiOperation("查询近30天工地视频感知事件")
    public Result selectByMonthEvent() {
        List<EventDto> events = buildingRecordsMapper.selectByMonthEvent();
        events.forEach(eventDto -> {
            eventDto.setEventNewName(eventDto.getEventCnName());
        });
        Map<String, Object> map = new HashMap<>();
        map.put("sum", events.size());
        map.put("data", events);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询近30天物联感知事件时间趋势
     *
     * @return
     */
    @GetMapping("selectMonthCurrentByBuilding")
    @ApiOperation("查询近30天物联感知事件时间趋势")
    public Result selectMonthCurrentByBuilding() {
        List<BuildingRecordsDtos> buildingRecordsDtos = buildingRecordsMapper.selectMonthCurrentByBuilding();
        buildingRecordsDtos.forEach(buildingRecordsDto -> {
            if (buildingRecordsDto.getNumber() == null) {
                buildingRecordsDto.setNumber(0);
            }
        });
        Result success = Result.success(buildingRecordsDtos);
        return success;
    }

    /**
     * 查询近30天工地视频感知事件时间趋势
     *
     * @return
     */
    @GetMapping("selectMonthCurrentByEvent")
    @ApiOperation("查询近30天工地视频感知事件时间趋势")
    public Result selectMonthCurrentByEvent() {
        List<BuildingRecordsDtos> buildingRecordsDtos = buildingRecordsMapper.selectMonthCurrentByEvent();
        buildingRecordsDtos.forEach(buildingRecordsDto -> {
            if (buildingRecordsDto.getNumber() == null) {
                buildingRecordsDto.setNumber(0);
            }
        });
        Result success = Result.success(buildingRecordsDtos);
        return success;
    }

    /**
     * 查询走航监测top5全部道路污染事件
     *
     * @return
     */
    @GetMapping("selectRoadDataCountEvent")
    @ApiOperation("查询走航监测top5全部道路污染事件")
    public Result selectRoadDataCountEvent() {
        List<RoadDataDto> roadDataDtos = buildingRecordsMapper.selectRoadDataCountEvent();
        Integer integer = roadDataMapper.selectCountRoadData();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", integer);
        map.put("roadDataDtos", roadDataDtos);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询走航监测top5近7天道路污染事件
     *
     * @return
     */
    @GetMapping("selectRoadDataWeekEvent")
    @ApiOperation("查询走航监测top5近7天道路污染事件")
    public Result selectRoadDataWeekEvent() {
        List<RoadDataDto> roadDataDtos = buildingRecordsMapper.selectRoadDataWeekEvent();
        Integer integer = roadDataMapper.selectCountWeekRoadData();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", integer);
        map.put("roadDataDtos", roadDataDtos);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询走航监测top5近30天道路污染事件
     *
     * @return
     */
    @GetMapping("selectRoadDataMonthEvent")
    @ApiOperation("查询走航监测top5近30天道路污染事件")
    public Result selectRoadDataMonthEvent() {
        List<RoadDataDto> roadDataDtos = buildingRecordsMapper.selectRoadDataMonthEvent();
        Integer integer = roadDataMapper.selectCountMonthRoadData();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", integer);
        map.put("roadDataDtos", roadDataDtos);
        Result success = Result.success(map);
        return success;
    }

    /**
     * 查询近30天走航监测事件时间趋势
     *
     * @return
     */
    @GetMapping("selectMonthCurrentByRoadData")
    @ApiOperation("查询近30天走航监测事件时间趋势")
    public Result selectMonthCurrentByRoadData() {
        List<BuildingRecordsDtos> buildingRecordsDtos = buildingRecordsMapper.selectMonthCurrentByRoadData();
        buildingRecordsDtos.forEach(buildingRecordsDto -> {
            if (buildingRecordsDto.getNumber() == null) {
                buildingRecordsDto.setNumber(0);
            }
        });
        Result success = Result.success(buildingRecordsDtos);
        return success;
    }


}
