package com.hisense.monitormanage.controller;

import com.hisense.monitormanage.dto.*;
import com.hisense.monitormanage.entity.*;
import com.hisense.monitormanage.mapper.*;
import com.hisense.monitormanage.service.*;
import com.hisense.monitormanage.utils.LongLatUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("api/project")
@Api(tags = "视频监控")
public class Controller {

    @Autowired
    private ProjectMapper projectMapper;

    @Autowired
    private CameraMapper cameraMapper;

    @Autowired
    private ScenicMapper scenicMapper;

    @Autowired
    private MonitorService monitorService;

    @Autowired
    private CameraChannelMapper cameraChannelMapper;

    @Autowired
    private CameraOrgenizationMapper cameraOrgenMapper;

    @Autowired
    private CameraChannelService cameraChannelService;

    @Autowired
    private ChannelPictureMapper channelPictureMapper;

    @GetMapping("all")
    public Object all() {
        List<Project> projects = projectMapper.selectList(null);
        projects.forEach(project -> {
            List<Picture> pics = cameraMapper.selectPicByProjectId(project.getId());
            project.setPics(pics);
        });

        return projects;
    }

    /**
     * 推送给城管接口用的
     *
     * @return
     */
    @GetMapping("cameras")
    public Result cameras() {
        List<ChannelPictureDto> channelPictureDtos = channelPictureMapper.selectByProject();
        Result success = Result.success(channelPictureDtos);
        success.setCount(channelPictureDtos.size());
        return success;
    }

    /**
     * @return
     */
    @GetMapping("camera")
    public Result camera() {
        List<PictureDto> pictureDtos = channelPictureMapper.selectByPicture();
        Result success = Result.success(pictureDtos);
        return success;
    }

    /**
     * 景区人流识别
     *
     * @return
     */
    @GetMapping("scenicCamera")
    public Result scenicCamera() {
        List<ScenicCameraDto> scenicCameraDtos = scenicMapper.selectScenicCameras();
        Result success = Result.success(scenicCameraDtos);
        success.setCount(scenicCameraDtos.size());
        return success;
    }

    @GetMapping("capture")
    public Object capture() {
        monitorService.CameraScreenshot();

        return "finish capture";
    }

    @GetMapping("startScenic")
    public Object startScenic() {
        monitorService.startScenic();

        return "startScenic finish";
    }

    @GetMapping("capture1")
    public Object capture1() {
        monitorService.scenicSpotAndBathing1();

        return "finish capture";
    }


    /**
     * 查询所有摄像头
     *
     * @param
     * @return
     */
    @GetMapping("selectAll")
    @ApiOperation("查询所有摄像头")
    public String selectAll() {
        return cameraChannelService.selectAll();
    }

    /**
     * 根据channelCode查询摄像头详细信息
     *
     * @param channelCode
     * @return
     */
    @GetMapping("selectByChannelCode")
    @ApiOperation("根据摄像头编号查询摄像头详细信息")
    @ApiImplicitParam(name = "channelCode", value = "摄像头标识", required = true, dataType = "String")
    public Result selectByChannelCode(String channelCode) {

        List<CameraChannelDto1> list = cameraChannelMapper.selectByChannelCode(channelCode);

        Result success = Result.success(list);

        return success;


    }

    /**
     * 根据摄像头名字模糊查询摄像头
     *
     * @param channelName
     * @return
     */
    @GetMapping("selectByChannelName")
    @ApiOperation("根据摄像头名字模糊查询摄像头")
    @ApiImplicitParam(name = "channelName", value = "摄像头名称", required = true, dataType = "String")
    public Result selectByChannelName(String channelName) {

        List<CameraChannel> labels = cameraChannelMapper.selectByChannelName(channelName);

        Result success = Result.success(labels);

        return success;
    }

    /**
     * 根据parentId和checkStatus查询摄像头
     *
     * @param parentId
     * @return
     */
    @GetMapping("selectByParentId")
    @ApiOperation("根据摄像头的组织Id和异常状态查询摄像头")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "parentId", value = "摄像头的组织Id", required = true, dataType = "String"),
            @ApiImplicitParam(name = "checkStatus", value = "异常状态 1正常,0异常", required = true, dataType = "String")
    })
    public Result selectByParentId(String parentId, String checkStatus) {

        List<CameraChannel> list = cameraChannelMapper.selectByParentId(parentId, checkStatus);

        Result success = Result.success(list);

        return success;

    }

    /**
     * 根据标签查询对应摄像头
     *
     * @return
     */
    @GetMapping("selectLabel")
    @ApiOperation("根据标签查询对应摄像头")
    @ApiImplicitParam(name = "labelCode", value = "标签号", required = true, dataType = "String")
    public Result selectLabel(String labelCode) {

        List<ChannelLabelDto> channelLabelDtos = cameraChannelMapper.selectLabel(labelCode);

        for (ChannelLabelDto channelLabelDto : channelLabelDtos) {
            String nodeName = channelLabelDto.getNodeName();
            String[] splits = nodeName.split("->");
            String split = splits[1];
            channelLabelDto.setNodeName(split);

        }

        return Result.success(channelLabelDtos);
    }

    /**
     * 根据经纬度查询半径内所有的摄像头
     *
     * @param gpsX
     * @param gpsY
     * @param radius
     * @return
     */
    @GetMapping("selectAllByGps")
    @ApiOperation("根据经纬度查询半径内所有的摄像头")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "gpsX", value = "经度", required = true, dataType = "Double"),
            @ApiImplicitParam(name = "gpsY", value = "纬度", required = true, dataType = "Double"),
            @ApiImplicitParam(name = "radius", value = "半径，米", required = true, dataType = "Integer"),}
    )
    public Result selectAllByGps(Double gpsX, Double gpsY, Integer radius) {
        double[] around = LongLatUtil.getAround(gpsX, gpsY, radius);
        List<CameraChannel> c = cameraChannelMapper.selectAllByGps(around[0], around[2], around[1], around[3]);
        Result success = Result.success(c);
        return success;
    }

    /**
     * 查询所有标签
     *
     * @return
     */
    @GetMapping("selectAllLabel")
    @ApiOperation("查询所有标签")
    public Result selectAllLabel() {

        List<Label> labels = cameraChannelMapper.selectAllLabel();

        Result success = Result.success(labels);

        return success;
    }

    /**
     * 查询city为1的标签
     *
     * @return
     */
    @GetMapping("selectByLabelCity")
    @ApiOperation("查询city为1的标签")
    public Result selectByLabelCity() {

        List<Label> labels = cameraChannelMapper.selectByLabelCity();

        Result success = Result.success(labels);

        return success;
    }

    /**
     * 根据标签名字模糊查询标签
     *
     * @param labelName
     * @return
     */
    @GetMapping("selectByLabelName")
    @ApiOperation("根据标签名字模糊查询标签")
    @ApiImplicitParam(name = "labelName", value = "标签名称", required = true, dataType = "String")
    public Result selectByLabelName(String labelName) {

        List<Label> labels = cameraChannelMapper.selectByLabelName(labelName);

        Result success = Result.success(labels);

        return success;
    }

    /**
     * 根据parentId查询组织
     *
     * @param parentId
     * @return
     */
    @GetMapping("selectSubOrgan")
    @ApiOperation("根据上级组织id查询组织")
    @ApiImplicitParam(name = "parentId", value = "上级组织id", required = true, dataType = "String")
    public Result selectSubOrganization(String parentId) {
        if (parentId == null) {
            parentId = "";
        }

        List<CameraOrganization> organs = cameraOrgenMapper.selectSubOrganization(parentId);

        Result success = Result.success(organs);

        return success;
    }

    /**
     * 提供给其他项目分页查询摄像头接口
     *
     * @param page
     * @param pageSize
     * @return
     */
    @GetMapping("selectCamera")
    @ApiOperation("提供给其他项目分页查询摄像头接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "页码", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页码大小", required = true, dataType = "Integer")
    })
    public Result selectNLAll(Integer page, Integer pageSize) {
        List<CameraChannelNLDto> selectNLAll = cameraChannelMapper.selectNLAll(page, pageSize);
        Result success = Result.success(selectNLAll);
        return success;
    }


}
