package com.hisense.monitormanage.controller;

import com.alibaba.fastjson.JSONObject;
import com.hisense.monitormanage.dto.EventDto;
import com.hisense.monitormanage.entity.Event;
import com.hisense.monitormanage.entity.Result;
import com.hisense.monitormanage.mapper.EventMapper;
import com.hisense.monitormanage.service.EventService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("STapi/project")
@Api(tags = "事件")
@Log4j2
public class EventController {

    @Autowired
    private EventService eventService;

    @Autowired
    private EventMapper eventMapper;

    /**
     * 订阅任务下发接口
     *
     * @return
     */
    @GetMapping("subscribe")
    public JSONObject subscribe() {
        return eventService.subscribe();
    }


    /**
     * 订阅任务下发接口
     *
     * @param request
     * @return
     * @throws IOException
     */
    @PostMapping("receive")
    @ApiOperation("订阅任务下发接口")
    public Result receive(HttpServletRequest request) throws IOException {
        ServletInputStream ris = request.getInputStream();
        StringBuilder content = new StringBuilder();
        byte[] b = new byte[1024];
        int lens = -1;
        while ((lens = ris.read(b)) > 0) {
            content.append(new String(b, 0, lens));
        }
        String strcont = content.toString();// 内容

        JSONObject jsonObject = JSONObject.parseObject(strcont);
        Event event = new Event();
        event.setEventCnName(jsonObject.getString("eventAlias"));
        event.setEventSerial(jsonObject.getString("eventSerial"));
        JSONObject attributes = jsonObject.getJSONObject("attributes");
        if (attributes != null) {
            JSONObject text = attributes.getJSONObject("text");
            if (text != null) {
                event.setVehicle(text.getString("valueDescription"));
            }
            JSONObject color = attributes.getJSONObject("color");
            if (color != null) {
                event.setColor(color.getString("valueDescription"));
            }
            JSONObject largeVehicle = attributes.getJSONObject("large_vehicle");
            if (largeVehicle != null) {
                event.setVehicle(largeVehicle.getString("valueDescription"));
            }
            JSONObject trainingVehicle = attributes.getJSONObject("training_vehicle");
            if (trainingVehicle != null) {
                event.setVehicle(trainingVehicle.getString("valueDescription"));
            }
            JSONObject smallVehicle = attributes.getJSONObject("small_vehicle");
            if (smallVehicle != null) {
                event.setVehicle(smallVehicle.getString("valueDescription"));
            }
            JSONObject aClass = attributes.getJSONObject("class");
            if (aClass != null) {
                event.setClasses(aClass.getString("valueDescription"));
            }
        }
        JSONObject camera = jsonObject.getJSONObject("camera");
        if (camera != null) {
            event.setCameraName(camera.getString("cameraName"));
            event.setDistrict(camera.getString("district"));
            event.setLatitude(camera.getBigDecimal("latitude"));
            event.setLongitude(camera.getBigDecimal("longitude"));
        }
        JSONObject image = jsonObject.getJSONObject("image");
        if (image != null) {
            event.setImageUrl(image.getString("imageUrl"));
        }
        event.setTaskId(jsonObject.getString("taskId"));
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");//要转换的时间格式
        Date date;
        try {
            date = sdf.parse(sdf.format(jsonObject.get("captureTime")));
            event.setCaptureTime(sdf.format(date));
        } catch (Exception e) {
            e.printStackTrace();
        }
        event.setTrackEvent(jsonObject.getString("trackEvent"));
        eventMapper.insert(event);
        log.info(event);
        return Result.success(strcont);
    }


    /**
     * 事件模板分页查询接口
     *
     * @return
     */
    @GetMapping("tamplate")
    @ApiOperation("事件模板分页查询接口")
    public JSONObject tamplate() {
        return eventService.template();

    }

    /**
     * 查询所有事件
     *
     * @return
     */
    @GetMapping("selectEvent")
    @ApiOperation("查询所有事件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "页码", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页码大小", required = false, dataType = "Integer")
    })
    public Result selectEvent(Integer page, Integer pageSize) {
        List<EventDto> events = eventMapper.selectEvent(page, pageSize);
        for (EventDto event : events) {
            if (event.getEventCnName().equals("街头伞篷") || event.getEventCnName().equals("水果和食品摊")
                    || event.getEventCnName().equals("地摊") || event.getEventCnName().equals("户外桌椅")) {
                event.setEventNewName("占道经营");
            } else if (event.getEventCnName().equals("纸箱") || event.getEventCnName().equals("街头散落垃圾")
                    || event.getEventCnName().equals("塑料篮子")) {
                event.setEventNewName("环境卫生");
            } else if (event.getEventCnName().equals("沿街晾晒衣物被单")) {
                event.setEventNewName("沿街晾晒");
            } else if (event.getEventCnName().equals("机动车违停")) {
                event.setEventNewName("违章停车");
            } else {
                event.setEventNewName(event.getEventCnName());
            }
        }
        Integer integer = eventMapper.selectCountEvent();
        page = integer / pageSize + 1;
        Map<String, Object> map = new HashMap<>();
        map.put("page", page);
        map.put("events", events);
        Result success = Result.success(map);

        return success;
    }

    /**
     * 查询当天事件
     *
     * @return
     */
    @GetMapping("selectByTime")
    @ApiOperation("查询当天事件")
    public Result selectByTime() {
        List<EventDto> events = eventMapper.selectByTime();
        for (EventDto event : events) {
            if (event.getEventCnName().equals("街头伞篷") || event.getEventCnName().equals("水果和食品摊")
                    || event.getEventCnName().equals("地摊") || event.getEventCnName().equals("户外桌椅")) {
                event.setEventNewName("占道经营");
            } else if (event.getEventCnName().equals("纸箱") || event.getEventCnName().equals("街头散落垃圾")
                    || event.getEventCnName().equals("塑料篮子")) {
                event.setEventNewName("环境卫生");
            } else if (event.getEventCnName().equals("沿街晾晒衣物被单")) {
                event.setEventNewName("沿街晾晒");
            } else if (event.getEventCnName().equals("机动车违停")) {
                event.setEventNewName("违章停车");
            } else {
                event.setEventNewName(event.getEventCnName());
            }
        }

        Result success = Result.success(events);

        return success;

    }

    /**
     * 根据事件名称查询事件
     *
     * @param eventNewName
     * @return
     */
    @GetMapping("selectByName")
    @ApiOperation("根据事件名称查询事件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "eventNewName", value = "事件名称", required = true, dataType = "String"),
            @ApiImplicitParam(name = "page", value = "页码", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页码大小", required = false, dataType = "Integer")
    })
    public Result selectByName(String eventNewName, Integer page, Integer pageSize) {
        if (eventNewName.equals("环境卫生")) {
            List<EventDto> lists = eventMapper.selectByName("'塑料篮子','纸箱','街头散落垃圾'", page, pageSize);
            lists.forEach(list -> {
                list.setEventNewName("环境卫生");
            });
            Integer integer = eventMapper.selectCountByName("'塑料篮子','纸箱','街头散落垃圾'");
            Map<String, Object> map = new HashMap<>();
            map.put("sum", integer);
            map.put("events", lists);
            Result success = Result.success(map);
            return success;
        } else if (eventNewName.equals("占道经营")) {
            List<EventDto> lists = eventMapper.selectByName("'街头伞篷','水果和食品摊','地摊','户外桌椅'", page, pageSize);
            lists.forEach(list -> {
                list.setEventNewName("占道经营");
            });
            Integer integer = eventMapper.selectCountByName("'街头伞篷','水果和食品摊','地摊','户外桌椅'");
            Map<String, Object> map = new HashMap<>();
            map.put("sum", integer);
            map.put("events", lists);
            Result success = Result.success(map);
            return success;
        } else if (eventNewName.equals("沿街晾晒")) {
            List<EventDto> lists = eventMapper.selectByName("'沿街晾晒衣物被单'", page, pageSize);
            lists.forEach(list -> {
                list.setEventNewName("沿街晾晒");
            });
            Integer integer = eventMapper.selectCountByName("'沿街晾晒衣物被单'");
            Map<String, Object> map = new HashMap<>();
            map.put("sum", integer);
            map.put("events", lists);
            Result success = Result.success(map);
            return success;
        } else if (eventNewName.equals("违章停车")) {
            List<EventDto> lists = eventMapper.selectByName("'机动车违停'", page, pageSize);
            lists.forEach(list -> {
                list.setEventNewName("违章停车");
            });
            Integer integer = eventMapper.selectCountByName("'机动车违停'");
            Map<String, Object> map = new HashMap<>();
            map.put("sum", integer);
            map.put("events", lists);
            Result success = Result.success(map);
            return success;
        } else {
            StringBuilder str = new StringBuilder(eventNewName);
            String s = str.insert(0, "'").append("'").toString();
            List<EventDto> lists = eventMapper.selectByName(s, page, pageSize);
            lists.forEach(list -> {
                list.setEventNewName(list.getEventCnName());
            });
            Integer integer = eventMapper.selectCountByName(s);
            Map<String, Object> map = new HashMap<>();
            map.put("sum", integer);
            map.put("events", lists);
            Result success = Result.success(map);
            return success;
        }
    }

    /**
     * 查询当天根据事件名称查询事件
     *
     * @param eventNewName
     * @return
     */
    @GetMapping("selectByTimeAndName")
    @ApiOperation("查询当天根据事件名称查询事件")
    @ApiImplicitParam(name = "eventNewName", value = "事件名称", required = true, dataType = "String")
    public Result selectByTimeAndName(String eventNewName) {
        if (eventNewName.equals("环境卫生")) {
            List<EventDto> lists = eventMapper.selectByTimeAndName("'塑料篮子','纸箱','街头散落垃圾'");
            for (EventDto list : lists) {
                list.setEventNewName("环境卫生");
            }
            Result success = Result.success(lists);
            return success;
        } else if (eventNewName.equals("占道经营")) {
            List<EventDto> lists = eventMapper.selectByTimeAndName("'街头伞篷','水果和食品摊','地摊','户外桌椅'");
            for (EventDto list : lists) {
                list.setEventNewName("占道经营");
            }
            Result success = Result.success(lists);
            return success;
        } else if (eventNewName.equals("沿街晾晒")) {
            List<EventDto> lists = eventMapper.selectByTimeAndName("'沿街晾晒衣物被单'");
            for (EventDto list : lists) {
                list.setEventNewName("沿街晾晒");
            }
            Result success = Result.success(lists);
            return success;
        } else if (eventNewName.equals("违章停车")) {
            List<EventDto> lists = eventMapper.selectByTimeAndName("'机动车违停'");
            for (EventDto list : lists) {
                list.setEventNewName("违章停车");
            }
            Result success = Result.success(lists);
            return success;
        } else {
            StringBuilder str = new StringBuilder(eventNewName);
            String s = str.insert(0, "'").append("'").toString();
            List<EventDto> lists = eventMapper.selectByTimeAndName(s);
            lists.forEach(list -> {
                list.setEventNewName(list.getEventCnName());
            });
            Result success = Result.success(lists);
            return success;
        }
    }
}
