package com.hisense.monitormanage.controller;


import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hisense.monitormanage.entity.CrowdFlow;
import com.hisense.monitormanage.entity.Result;
import com.hisense.monitormanage.mapper.CrowdFlowMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

@RestController
@RequestMapping("api/gaode")
@Api(tags = "高德类数据")
public class GaodeController {

    @Autowired
    private CrowdFlowMapper crowdFlowMapper;

    @Autowired
    private RestTemplate restTemplate;

    @Value("${gaode.url}")
    private String gaodeUrl;

    @GetMapping("crowflow/list")
    @ApiOperation("查询不同时间客流量")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "页码", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "size", value = "页码大小", required = false, dataType = "Integer")
    })
    public Result crowflowList(Integer page, Integer size,
                               String start, String end,
                               Integer cValue,
                               Boolean needAreaData) {

        LambdaQueryWrapper<CrowdFlow> queryWrapper = new QueryWrapper<CrowdFlow>().lambda();
        if (start != null) {
            queryWrapper.gt(CrowdFlow::getCaptureTime, start);
        }
        if (end != null) {
            queryWrapper.le(CrowdFlow::getCaptureTime, end);
        }
        if (cValue != null && cValue > 0) {
            queryWrapper.gt(CrowdFlow::getCnt, cValue);
        }

        if (needAreaData == null || needAreaData == false) {
            queryWrapper.select(CrowdFlow::getId,
                    CrowdFlow::getCaptureTime,
                    CrowdFlow::getCnt,
                    CrowdFlow::getVisitCnt);
        }

        Page<CrowdFlow> flowPage = new Page<>();
        flowPage.addOrder(OrderItem.desc("id"));
        if (page != null && page > 0) {
            flowPage.setCurrent(page);
        }
        if (size != null && size > 0) {
            flowPage.setSize(size);
        }
        Page<CrowdFlow> result = crowdFlowMapper.selectPage(flowPage, queryWrapper);

        Result success = Result.success(result.getRecords());
        success.setCount((int) result.getTotal());
        return success;
    }

    /**
     * 手动刷一条数据
     * @param date 格式yyyyMMddHHmm 202211261700
     */
    @GetMapping("fetchGaodeData")
    public void fetchGaodeDataByHand(String date) throws InterruptedException {
        Date currentDate = new Date();
        if (date != null) {
            currentDate = DateUtil.parse(date, "yyyyMMddHHmm");
        }
        Integer cnt = _fetchGaodeData(currentDate, "total_flow");
        Thread.sleep(1 * 1000);
        Integer visitCnt = _fetchGaodeData(currentDate, "visit_flow");
        Thread.sleep(1 * 1000);
        String total_flow = _fetchGaodeHotData(currentDate, "total_flow");
        Thread.sleep(1 * 1000);
        String visit_flow = _fetchGaodeHotData(currentDate, "visit_flow");
        LambdaQueryWrapper<CrowdFlow> queryWrapper = new QueryWrapper<CrowdFlow>().lambda()
                .eq(CrowdFlow::getCaptureTime, currentDate);
        CrowdFlow entity = crowdFlowMapper.selectOne(queryWrapper);
        if (entity == null){
            CrowdFlow crowdFlow = new CrowdFlow();
            crowdFlow.setCaptureTime(currentDate);
            crowdFlow.setCnt(cnt);
            crowdFlow.setVisitCnt(visitCnt);
            crowdFlowMapper.insert(crowdFlow);
        }else {
            entity.setCnt(cnt);
            entity.setVisitCnt(visitCnt);
            entity.setTotalAreaData(total_flow);
            entity.setVisitAreaData(visit_flow);
            crowdFlowMapper.updateById(entity);
        }
    }

    /**
     * 刷一遍所有数据
     *
     */
    @GetMapping("fixGaodeData")
    public void fixGaodeDataByHand() throws InterruptedException {

        Page<CrowdFlow> flowPage = new Page<>();
        flowPage.addOrder(OrderItem.asc("id"));
        Page<CrowdFlow> crowdFlowPage = null;
        do {

            crowdFlowPage = crowdFlowMapper.selectPage(flowPage, null);
            List<CrowdFlow> crowdFlows = crowdFlowPage.getRecords();
            for (CrowdFlow crowdFlow : crowdFlows) {
                //限流策略， 每秒不超过3次。每分钟不超过100次，每小时不超过1000次
                Integer cnt = _fetchGaodeData(crowdFlow.getCaptureTime(), "total_flow");
                Thread.sleep(3 * 1000);
                Integer visitCnt = _fetchGaodeData(crowdFlow.getCaptureTime(), "visit_flow");
                //接口有限流，调一会就不通了,延时一会
                Thread.sleep(3 * 1000);
                String total_flow = _fetchGaodeHotData(crowdFlow.getCaptureTime(), "total_flow");
                Thread.sleep(3 * 1000);
                String visit_flow = _fetchGaodeHotData(crowdFlow.getCaptureTime(), "visit_flow");
                if (crowdFlow.getVisitCnt() != visitCnt || crowdFlow.getCnt() != cnt ||
                        crowdFlow.getVisitAreaData() == null ||
                        crowdFlow.getTotalAreaData() == null) {
                    crowdFlow.setCnt(cnt);
                    crowdFlow.setVisitCnt(visitCnt);
                    crowdFlow.setTotalAreaData(total_flow);
                    crowdFlow.setVisitAreaData(visit_flow);
                    crowdFlowMapper.updateById(crowdFlow);
                }
            }
            flowPage.setCurrent(flowPage.getCurrent()+1);

        }while (crowdFlowPage.hasNext());
    }

    public Integer _fetchGaodeData(Date currentDate, String flowType) {
        StringBuilder url = new StringBuilder(gaodeUrl + "/v4/biolap/abapi/query");
        url.append('?');
        TreeMap<String, Object> parame = new TreeMap<>();
        parame.put("key","a7c2748d8af3393b6fa0531023d2b8c8");

        parame.put("appname","dz-qingdao-chengguan");
        parame.put("apiid","490");
        parame.put("areaId","WY00017X7U");
        parame.put("timeType","15m");
        parame.put("flowType", flowType);
        String ds = DateUtil.format(currentDate, "yyyyMMddHHmm");
        parame.put("ds", ds);
        parame.put("timestamp", new Date().getTime() / 1000);
        StringBuilder builder = new StringBuilder();
        parame.forEach((key, value) -> {
            builder.append(value + ";");
            url.append(key + "={" + key + "}&");
        });
        String secretkey = "4c32a89db1f20b222e08919b29567368";
        String token = SecureUtil.md5(builder.toString() + secretkey);
        parame.put("token", token);
        url.append("token={token}");
        HashMap forObject = restTemplate.getForObject(url.toString(), HashMap.class, parame);
        System.out.println(JSON.toJSONString(forObject));
        if ((Integer) forObject.get("errcode") == 0) {
            Map data = (Map) forObject.get("data");
            if (data != null) {
                Map indexData = (Map) data.get("index_data");
                List<Map> query = (List) indexData.get("query");
                if (query != null && query.size() == 1) {
                    return (Integer) query.get(0).get("index");
                }
            }


        }

        return null;
    }

    public String _fetchGaodeHotData(Date currentDate, String flowType) {
        StringBuilder url = new StringBuilder(gaodeUrl + "/v4/biolap/abapi/query");
        url.append('?');
        TreeMap<String, Object> parame = new TreeMap<>();
        parame.put("key","a7c2748d8af3393b6fa0531023d2b8c8");

        parame.put("appname","dz-qingdao-chengguan");
        parame.put("apiid","484");
//        parame.put("areaId","WY00017X7U");
        parame.put("areaId","WY00017X7V");
        parame.put("timeType","15m");
        parame.put("flowType", flowType);
        String ds = DateUtil.format(currentDate, "yyyyMMddHHmm");
        parame.put("ds", ds);
        parame.put("geoLength", 10);
        parame.put("timestamp", new Date().getTime() / 1000);
        StringBuilder builder = new StringBuilder();
        parame.forEach((key, value) -> {
            builder.append(value + ";");
            url.append(key + "={" + key + "}&");
        });
        String secretkey = "4c32a89db1f20b222e08919b29567368";
        String token = SecureUtil.md5(builder.toString() + secretkey);
        parame.put("token", token);
        url.append("token={token}");
        HashMap forObject = restTemplate.getForObject(url.toString(), HashMap.class, parame);
        System.out.println(JSON.toJSONString(forObject));
        if ((Integer) forObject.get("errcode") == 0) {
            Map data = (Map) forObject.get("data");
            if (data != null) {
                Map indexData = (Map) data.get("index_data");
                Map analysis = (Map) indexData.get("query");
                if (analysis != null && analysis.get("area_data") != null) {
                    return JSON.toJSONString(analysis.get("area_data"));
                }
            }


        }

        return null;
    }



    /**
     * 从0开始每隔15分钟执行一次，也就是0,15,30,45
     * 为了查询准确需要查上一个15分钟
     */
    @Scheduled(cron = "0 0/15 * * * ?")
    public void fetchGaodeData() {

        DateTime dateTime = DateUtil.date().setField(DateField.MILLISECOND, 0);
        DateTime currentDate = DateUtil.offsetMinute(dateTime, -15);

        Integer cnt = _fetchGaodeData(currentDate, "total_flow");
        if (cnt == null){
            cnt = 0;
        }

        Integer visitCnt = _fetchGaodeData(currentDate, "visit_flow");
        if (visitCnt == null) {
            visitCnt = 0;
        }

        String total_flow = _fetchGaodeHotData(currentDate, "total_flow");
        String visit_flow = _fetchGaodeHotData(currentDate, "visit_flow");
        CrowdFlow crowdFlow = new CrowdFlow();
        crowdFlow.setCaptureTime(currentDate);
        crowdFlow.setCnt(cnt);
        crowdFlow.setVisitCnt(visitCnt);
        crowdFlow.setVisitAreaData(visit_flow);
        crowdFlow.setTotalAreaData(total_flow);
        crowdFlowMapper.insert(crowdFlow);

    }
}
