package com.hisense.monitormanage.controller;

import com.hisense.monitormanage.dto.EventDto;
import com.hisense.monitormanage.dto.KeyAreaDto;
import com.hisense.monitormanage.dto.KeyAreaDtos;
import com.hisense.monitormanage.entity.KeyArea;
import com.hisense.monitormanage.entity.KeyAreaLocation;
import com.hisense.monitormanage.entity.PassengerFlow;
import com.hisense.monitormanage.entity.Result;
import com.hisense.monitormanage.mapper.EventMapper;
import com.hisense.monitormanage.mapper.KeyAreaMapper;
import com.hisense.monitormanage.mapper.PassengerFlowMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("api/project/keyArea")
@Api(tags = "重点区域")
public class KeyAreaController {

    @Autowired
    private KeyAreaMapper keyAreaMapper;

    @Autowired
    private PassengerFlowMapper passengerFlowMapper;

    @Autowired
    private EventMapper eventMapper;

    /**
     * 根据区域类型查询全部区域列表的最新时间的人数
     *
     * @param type1
     * @param type2
     * @param type3
     * @return
     */
    @GetMapping("selectKeyAreaList")
    @ApiOperation("根据区域类型查询全部区域列表的最新时间的人数")
    public Result selectKeyAreaList(String type1, String type2, String type3) {
        PassengerFlow maxId = passengerFlowMapper.getByMaxId();
        String timeId = maxId.getTimeId();
        List<KeyAreaDto> keyAreaDtos = keyAreaMapper.selectKeyAreaList(type1, type2, type3, timeId);
        keyAreaDtos.forEach(keyAreaDto -> {
            keyAreaDto.setTimeId(timeId);
        });
        Result success = Result.success(keyAreaDtos);
        return success;
    }

    /**
     * 根据区域类型查询区域的最新时间的人数
     *
     * @param type
     * @return
     */
    @GetMapping("selectKeyArea")
    @ApiOperation("根据区域类型查询区域的最新时间的人数")
    public Result selectKeyArea(String type, String area_name) {
        PassengerFlow maxId = passengerFlowMapper.getByMaxId();
        String timeId = maxId.getTimeId();
        List<KeyAreaDto> keyAreaDtos = keyAreaMapper.selectKeyArea(type, timeId);
        keyAreaDtos.forEach(keyAreaDto -> {
            keyAreaDto.setTimeId(timeId);
        });
        Result success = Result.success(keyAreaDtos);
        return success;
    }

    /**
     * 根据名称查询商圈范围
     *
     * @param areaName
     * @return
     */
    @GetMapping("selectByAreaName")
    @ApiOperation("根据名称查询商圈范围")
    public Result selectByAreaName(String areaName) {
        List<KeyAreaLocation> keyAreaLocations = keyAreaMapper.selectByAreaName(areaName);
        Result success = Result.success(keyAreaLocations);
        return success;
    }

    /**
     * 根据名称查询重点区域名称
     *
     * @param areaName
     * @return
     */
    @GetMapping("selectByLikeName")
    @ApiOperation("根据名称查询重点区域名称")
    @ApiImplicitParam(name = "areaName", value = "重点区域名称", required = true, dataType = "String")
    public Result selectByLikeName(String areaName) {
        List<KeyArea> keyAreaLocations = keyAreaMapper.selectByLikeName(areaName);
        Result success = Result.success(keyAreaLocations);
        return success;
    }

    /**
     * 重点区域场景查询所有事件
     *
     * @return
     */
    @GetMapping("selectByAreaEvent")
    @ApiOperation("重点区域场景查询所有事件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "页码", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页码大小", required = true, dataType = "Integer")
    })
    public Result selectByAreaEvent(Integer page, Integer pageSize) {
        List<EventDto> events = keyAreaMapper.selectByAreaEvent(page, pageSize);
        events.forEach(event -> {
            if (event.getEventCnName().equals("街头伞篷") || event.getEventCnName().equals("水果和食品摊")
                    || event.getEventCnName().equals("地摊") || event.getEventCnName().equals("户外桌椅")) {
                event.setEventNewName("占道经营");
            } else if (event.getEventCnName().equals("纸箱") || event.getEventCnName().equals("街头散落垃圾")
                    || event.getEventCnName().equals("塑料篮子")) {
                event.setEventNewName("环境卫生");
            } else if (event.getEventCnName().equals("沿街晾晒衣物被单")) {
                event.setEventNewName("沿街晾晒");
            } else if (event.getEventCnName().equals("机动车违停")) {
                event.setEventNewName("违章停车");
            } else {
                event.setEventNewName(event.getEventCnName());
            }
        });
        Integer integer = keyAreaMapper.selectCountAreaEvent();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", integer);
        map.put("events", events);
        Result success = Result.success(map);

        return success;
    }

    /**
     * 重点区域场景查询当天事件
     *
     * @return
     */
    @GetMapping("selectByDayAreaEvent")
    @ApiOperation("重点区域场景查询当天事件")
    public Result selectByDayAreaEvent() {
        List<EventDto> events = keyAreaMapper.selectByDayAreaEvent();
        events.forEach(event -> {
            if (event.getEventCnName().equals("街头伞篷") || event.getEventCnName().equals("水果和食品摊")
                    || event.getEventCnName().equals("地摊") || event.getEventCnName().equals("户外桌椅")) {
                event.setEventNewName("占道经营");
            } else if (event.getEventCnName().equals("纸箱") || event.getEventCnName().equals("街头散落垃圾")
                    || event.getEventCnName().equals("塑料篮子")) {
                event.setEventNewName("环境卫生");
            } else if (event.getEventCnName().equals("沿街晾晒衣物被单")) {
                event.setEventNewName("沿街晾晒");
            } else if (event.getEventCnName().equals("机动车违停")) {
                event.setEventNewName("违章停车");
            } else {
                event.setEventNewName(event.getEventCnName());
            }
        });
        Map<String, Object> map = new HashMap<>();
        map.put("sum", events.size());
        map.put("events", events);
        Result success = Result.success(map);
        return success;

    }

    /**
     * 重点区域场景根据事件名称查询事件
     *
     * @param eventNewName
     * @return
     */
    @GetMapping("selectByNameAreaEvent")
    @ApiOperation("重点区域场景根据事件名称查询事件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "eventNewName", value = "事件名称", required = true, dataType = "String"),
            @ApiImplicitParam(name = "page", value = "页码", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "页码大小", required = false, dataType = "Integer")
    })
    public Result selectByNameAreaEvent(String eventNewName, Integer page, Integer pageSize) {
        if (eventNewName.equals("环境卫生")) {
            List<EventDto> lists = keyAreaMapper.selectByNameAreaEvent("'塑料篮子','纸箱','街头散落垃圾'", page, pageSize);
            lists.forEach(list -> {
                list.setEventNewName("环境卫生");
            });
            Integer integer = keyAreaMapper.selectCountByNameAreaEvent("'塑料篮子','纸箱','街头散落垃圾'");
            Map<String, Object> map = new HashMap<>();
            map.put("sum", integer);
            map.put("events", lists);
            Result success = Result.success(map);
            return success;
        } else if (eventNewName.equals("占道经营")) {
            List<EventDto> lists = keyAreaMapper.selectByNameAreaEvent("'街头伞篷','水果和食品摊','地摊','户外桌椅'", page, pageSize);
            lists.forEach(list -> {
                list.setEventNewName("占道经营");
            });
            Integer integer = keyAreaMapper.selectCountByNameAreaEvent("'街头伞篷','水果和食品摊','地摊','户外桌椅'");
            Map<String, Object> map = new HashMap<>();
            map.put("sum", integer);
            map.put("events", lists);
            Result success = Result.success(map);
            return success;
        } else {
            StringBuilder str = new StringBuilder(eventNewName);
            String s = str.insert(0, "'").append("'").toString();
            List<EventDto> lists = keyAreaMapper.selectByNameAreaEvent(s, page, pageSize);
            lists.forEach(list -> {
                list.setEventNewName(list.getEventCnName());
            });
            Integer integer = keyAreaMapper.selectCountByNameAreaEvent(s);
            Map<String, Object> map = new HashMap<>();
            map.put("sum", integer);
            map.put("events", lists);
            Result success = Result.success(map);
            return success;
        }
    }

    /**
     * 查询近30天重点区域视频感知事件时间趋势
     *
     * @return
     */
    @GetMapping("selectMonthAreaEvent")
    @ApiOperation("查询近30天重点区域视频感知事件时间趋势")
    public Result selectMonthAreaEvent() {
        List<KeyAreaDtos> keyAreaDtos = keyAreaMapper.selectMonthAreaEvent();
        keyAreaDtos.forEach(keyAreaDtos1 -> {
            if (keyAreaDtos1.getNumber() == null) {
                keyAreaDtos1.setNumber(0);
            }
        });
        Result success = Result.success(keyAreaDtos);
        return success;
    }

    /**
     * 统计事件类型
     *
     * @return
     */
    @GetMapping("selectEventType")
    @ApiOperation("统计事件类型")
    public Result selectEventType() {
        List<KeyAreaDtos> keyAreaDtos = keyAreaMapper.selectEventType();
        keyAreaDtos.forEach(keyAreaDtos1 -> {
            if (keyAreaDtos1.getEventCnName().equals("街头伞篷") || keyAreaDtos1.getEventCnName().equals("水果和食品摊")
                    || keyAreaDtos1.getEventCnName().equals("地摊") || keyAreaDtos1.getEventCnName().equals("户外桌椅")) {
                keyAreaDtos1.setEventNewName("占道经营");
            } else if (keyAreaDtos1.getEventCnName().equals("纸箱") || keyAreaDtos1.getEventCnName().equals("街头散落垃圾")
                    || keyAreaDtos1.getEventCnName().equals("塑料篮子")) {
                keyAreaDtos1.setEventNewName("环境卫生");
            } else if (keyAreaDtos1.getEventCnName().equals("沿街晾晒衣物被单")) {
                keyAreaDtos1.setEventNewName("沿街晾晒");
            } else if (keyAreaDtos1.getEventCnName().equals("机动车违停")) {
                keyAreaDtos1.setEventNewName("违章停车");
            } else {
                keyAreaDtos1.setEventNewName(keyAreaDtos1.getEventCnName());
            }
        });
        Result success = Result.success(keyAreaDtos);
        return success;
    }

    /**
     * 查询近7天重点区域本地外地人流峰值按每一天输出
     *
     * @return
     */
    @GetMapping("selectLocalWeekArea")
    @ApiOperation("查询近7天重点区域本地外地人流峰值按每一天输出")
    public Result selectLocalWeekArea(String areaName) {
        List<KeyAreaDtos> keyAreaDtos = keyAreaMapper.selectLocalWeekArea(areaName);
        keyAreaDtos.forEach(keyAreaDtos1 -> {
            if (keyAreaDtos1.getLocalNums() == null && keyAreaDtos1.getNonlocalNums() == null) {
                keyAreaDtos1.setLocalNums(0);
                keyAreaDtos1.setNonlocalNums(0);
            }
        });
        Result success = Result.success(keyAreaDtos);
        return success;
    }

    /**
     * 查询近30天重点区域本地外地人流峰值按每一天输出
     *
     * @return
     */
    @GetMapping("selectLocalMonthArea")
    @ApiOperation("查询近30天重点区域本地外地人流峰值按每一天输出")
    public Result selectLocalMonthArea(String areaName) {
        List<KeyAreaDtos> keyAreaDtos = keyAreaMapper.selectLocalMonthArea(areaName);
        keyAreaDtos.forEach(keyAreaDtos1 -> {
            if (keyAreaDtos1.getLocalNums() == null && keyAreaDtos1.getNonlocalNums() == null) {
                keyAreaDtos1.setLocalNums(0);
                keyAreaDtos1.setNonlocalNums(0);
            }
        });
        Result success = Result.success(keyAreaDtos);
        return success;
    }

    /**
     * 查询重点区域抓图照片
     *
     * @return
     */
    @GetMapping("selectByAreaImage")
    @ApiOperation("查询重点区域抓图照片")
    public Result selectByAreaImage(String areaName) {
        List<KeyAreaDto> keyAreaDtos = keyAreaMapper.selectByAreaImage(areaName);
        Result success = Result.success(keyAreaDtos);
        return success;
    }


}
