package com.hisense.monitormanage.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.hisense.monitormanage.dto.PassengerFlowDto;
import com.hisense.monitormanage.dto.RoadDataDtos;
import com.hisense.monitormanage.entity.*;
import com.hisense.monitormanage.mapper.*;
import com.hisense.monitormanage.service.*;
import com.hisense.monitormanage.utils.LongLatUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("api/project")
@Api(tags = "启迪中台")
public class QidiController {
    @Autowired
    private SedimentTrailService sedimentTrailService;

    @Autowired
    private PassengerFlowService passengerFlowService;

    @Autowired
    private TrailSanitationService trailSanitationService;

    @Autowired
    private RoadDataService roadDataService;

    @Autowired
    private BuildingSiteService buildingSitrService;

    @Autowired
    private BuildingSiteMapper buildingSiteMapper;

    @Autowired
    private PassengerFlowMapper passengerFlowlMapper;

    @Autowired
    private TrailSanitationMapper trailSanitationMapper;

    @Autowired
    private QidiService qidiService;

    @Autowired
    private RoadDataMapper roadDataMapper;

    @Autowired
    private SedimentTrailMapper sedimentTrailMapper;

    //道路统计数据与排名
    @GetMapping("roadData")
    @ApiOperation("道路统计数据与排名，根据后台dt调用接口")
    public List<Map> roadData() {
        return roadDataService.roadData();
    }

    //启迪中台
    //登录
    @GetMapping("qidiToken")
    @ApiOperation("启迪数据中台登录获取token")
    public String qidiToken() {
        return qidiService.qidiToken();
    }

    //渣土车轨迹数据
    @GetMapping("resCatalogApplyZTYS")
    @ApiOperation("获取渣土车轨迹数据,根据后台UPLOADTIME调用接口")
    public List<Map> resCatalogApply() {
        return qidiService.resCatalogApplyZTYS();
    }

    //环卫车轨迹数据
    @GetMapping("resCatalogApplyHJWS")
    @ApiOperation("环卫车轨迹数据，根据后台updatetime调用接口")
    public List<Map> resCatalogApplyHJWS() {
        return qidiService.resCatalogApplyHJWS();
    }

    //工地信息
    @GetMapping("resCatalogApplyGDYS")
    @ApiOperation("获取工地信息-表中获取")
    public List<Map> resCatalogApplyGDYS() {
        return qidiService.resCatalogApplyGDYS();
    }

    //环卫车辆数据1，基础
    @GetMapping("resCatalogApplyHJWSBase")
    @ApiOperation("环卫车辆数据1,基础--根据后台updatetime调用接口")
    public List<Map> resCatalogApplyHJWSBase() {
        return qidiService.resCatalogApplyHJWSBase();
    }

    //环卫车辆数据1，作业
    @GetMapping("resCatalogApplyHJWSZY")
    @ApiOperation("环卫车辆数据1，作业--根据后台ZYRQ调用接口")
    public List<Map> resCatalogApplyHJWSZY() {
        return qidiService.resCatalogApplyHJWSZY();
    }

    //环卫道路明细数据
    @GetMapping("resCatalogApplyHJWSRoad")
    @ApiOperation("环卫道路明细数据,根据后台updatetime调用接口")
    public List<Map> resCatalogApplyHJWSRoad() {
        return qidiService.resCatalogApplyHJWSRoad();
    }

    //获取渣土车轨迹并保存到t_trail_sediment
    @GetMapping(value = "saveResCatalogApplyZTYS")
    @ApiOperation("取渣土车轨迹并保存到t_trail_sediment，根据后台UPLOADTIME调用接口")
    public Result resCatalogApplyZTYS() {

        boolean result = sedimentTrailService.batchSaveSedimentTrail();
        if (result) {
            return Result.success();
        } else {
            return Result.error(String.valueOf(result));
        }
    }

    //根据输入的经纬度和查询的半径（米）时间查询范围内的渣土车轨迹信息
    @GetMapping(value = "listSedimentTrailByPoints")
    @ApiOperation("渣土车轨迹信息，根据经纬度和半径从表中查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "longitude", value = "经度", paramType = "query", required = true, dataType = "double"),
            @ApiImplicitParam(name = "latitude", value = "纬度", paramType = "query", required = true, dataType = "double"),
            @ApiImplicitParam(name = "radius", value = "半径，米", paramType = "query", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "start", value = "开始时间，2021-08-03 10:17:23", paramType = "query", required = true, dataType = "string"),
            @ApiImplicitParam(name = "end", value = "结束时间，2021-08-03 12:17:23", paramType = "query", required = true, dataType = "string")
    })
    public Result listSedimentTrailByPoints(
            @RequestParam(value = "longitude") double longitude,
            @RequestParam(value = "latitude") double latitude,
            @RequestParam(value = "radius") Integer radius,
            @RequestParam(value = "start") String start,
            @RequestParam(value = "end") String end
    ) {
        List<SedimentTrail> map = sedimentTrailService.listSedimentTrailByPoints(longitude, latitude, radius, start, end);
        return Result.success(map);
    }


    //测试获取最新的客流列表，从接口获取
    @GetMapping("passengerFlow")
    @ApiOperation("获取最新的客流列表，测试接口，调用接口获取")
    public List<Map> passengerFlow() {
        List<Map> list = passengerFlowService.passengerFlow();
        return list;
    }

    //获取最新的实时客流列表，从表获取
    @GetMapping("listPassengerFlow")
    @ApiOperation("获取最新的实时客流列表，从表获取,热力图")
    @ApiImplicitParam(name = "timeId", value = "时间点", required = false, paramType = "query", dataType = "string")
    public List<PassengerFlow> listPassengerFlow(@RequestParam(value = "timeId", required = false, defaultValue = "1970000000") String timeId) {
        List<PassengerFlow> list = passengerFlowService.listPassengerFlow(timeId);
        return list;
    }

    //从表中获取最新的客流列表，并筛选出all_nums >=5000的
    @GetMapping("listPassengerFlowByAllNums")
    @ApiOperation("从表中获取最新的客流列表，并筛选出all_nums >=5000的")
    public List<PassengerFlow> listPassengerFlowByAllNums() {
        List<PassengerFlow> list = passengerFlowService.listPassengerFlowByAllNums();
        return list;
    }

    //获取实时客流并保存到表t_passenger_flow
    @GetMapping("savePassengerFlow")
    @ApiOperation("获取实时客流并保存到表t_passenger_flow,调用接口")
    public Result savePassengerFlow() {
        List<Map> list = passengerFlowService.passengerFlow();
        boolean result = passengerFlowService.savePassengerFlow(list);
        if (result) {
            return Result.success();
        } else {
            return Result.error(String.valueOf(result));
        }
    }

    //根据输入的经纬度和查询的半径（米）和timeId查询游客总数
    @GetMapping("getPassengerNum")
    @ApiOperation("查询游客总数，根据经纬度和半径米、timeId从表中查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "longitude", value = "经度", paramType = "query", required = false, dataType = "double"),
            @ApiImplicitParam(name = "latitude", value = "纬度", paramType = "query", required = false, dataType = "double"),
            @ApiImplicitParam(name = "radius", value = "半径，米", paramType = "query", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "timeId", value = "时间点，202205101600", paramType = "query", required = true, dataType = "string"),
    })
    public Result getPassengerNum(
            @RequestParam(value = "longitude", required = false, defaultValue = "0.00") double longitude,
            @RequestParam(value = "latitude", required = false, defaultValue = "0.00") double latitude,
            @RequestParam(value = "radius", required = false, defaultValue = "0") Integer radius,
            @RequestParam(value = "timeId", required = true) String timeId
    ) {
        return Result.success(passengerFlowService.passengerNums(longitude, latitude, radius, timeId));
    }

    /**
     * 查询半径范围内格栅
     *
     * @param longitude
     * @param latitude
     * @param radius
     * @param timeId
     * @return
     */
    @GetMapping("selectPassenger")
    @ApiOperation("查询半径范围内格栅，根据经纬度和半径米、timeId从表中查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "longitude", value = "经度", paramType = "query", required = false, dataType = "double"),
            @ApiImplicitParam(name = "latitude", value = "纬度", paramType = "query", required = false, dataType = "double"),
            @ApiImplicitParam(name = "radius", value = "半径，米", paramType = "query", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "timeId", value = "时间点，202205101600", paramType = "query", required = true, dataType = "string"),
    })
    public Result selectPassenger(Double longitude, Double latitude, Integer radius, String timeId) {
        double[] around = LongLatUtil.getAround(longitude, latitude, radius);
        List<PassengerFlowDto> c = passengerFlowlMapper.selectPassenger(around[0], around[2], around[1], around[3], timeId);
        Result success = Result.success(c);
        return success;
    }

    //查询并保存环卫车轨迹到表t_trail_sanitation
    @GetMapping("saveTrailSanitation")
    @ApiOperation("保存环卫车轨迹到表t_trail_sanitation,根据后台updatetime调用接口")
    public Result saveTrailSanitation() {
        boolean result = trailSanitationService.saveTrailSanitation();
        if (result) {
            return Result.success();
        } else {
            return Result.error(String.valueOf(result));
        }
    }

    //根据输入的经纬度和查询的半径（米）时间查询范围内的环卫车轨迹信息
    @GetMapping(value = "listTrailSanitationByPoints")
    @ApiOperation("查询范围内环卫车轨迹信息，根据经纬度、半径米，时间从表中查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "longitude", value = "经度", paramType = "query", required = true, dataType = "double"),
            @ApiImplicitParam(name = "latitude", value = "纬度", paramType = "query", required = true, dataType = "double"),
            @ApiImplicitParam(name = "radius", value = "半径，米", paramType = "query", required = true, dataType = "Integer"),
            @ApiImplicitParam(name = "start", value = "开始时间，2021-03-17 00:00:00", paramType = "query", required = true, dataType = "string"),
            @ApiImplicitParam(name = "end", value = "结束时间，2021-03-22 00:00:00", paramType = "query", required = true, dataType = "string"),
    })
    public Result listTrailSanitationByPoints(
            @RequestParam(value = "longitude") double longitude,
            @RequestParam(value = "latitude") double latitude,
            @RequestParam(value = "radius") Integer radius,
            @RequestParam(value = "start") String start,
            @RequestParam(value = "end") String end
    ) {
        List<TrailSanitation> list = trailSanitationService.listSedimentTrailByPoints(longitude, latitude, radius, start, end);
        return Result.success(list);
    }

    /**
     * 查询全部环卫车轨迹信息
     *
     * @return
     */
    @GetMapping("selectAllTrailSanitation")
    @ApiOperation("查询全部环卫车轨迹信息")
    public Result selectAllTrailSanitation() {
        List<TrailSanitation> trailSanitations = trailSanitationMapper.selectAllTrailSanitation();
        Result success = Result.success(trailSanitations);
        return success;
    }

    /**
     * 查询全部渣土车轨迹信息
     *
     * @return
     */
    @GetMapping("selectAllSedimentTrail")
    @ApiOperation("查询全部渣土车轨迹信息")
    public Result selectAllSedimentTrail() {
        List<SedimentTrail> trailSanitations = sedimentTrailMapper.selectAllSedimentTrail();
        Result success = Result.success(trailSanitations);
        return success;
    }

    //查询道路污染数据
    //查询道路污染数据,根据dt从表中最新时间的数据
    @GetMapping(value = "listRoadData")
    @ApiOperation("查询道路污染数据,根据dt从表中最新时间的数据")
    public Result listRoadData() {
        RoadData byMaxId = roadDataMapper.getByMaxId();
        String dt = byMaxId.getDt();
        List<RoadData> roadData = roadDataService.listRoadData(dt);
        roadData.forEach(roadData1 -> {
            roadData1.setDt(roadData1.getDt().substring(0, 10));
        });
        return Result.success(roadData);
    }

    //根据输入的日期间隔查询道路污染数据(例如：2022-05-10 00:00:00至2022-05-11 00:00:00）
    @GetMapping(value = "listRoadDataByTimeRegion")
    @ApiOperation("根据输入的日期间隔查询道路污染数据(例如：2022-05-10 00:00:00至2022-05-11 00:00:00）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "begin", value = "开始日期", paramType = "query", dataType = "string", required = true),
            @ApiImplicitParam(name = "over", value = "结束日期", paramType = "query", dataType = "string", required = true)
    })
    public Result listRoadDataByTimeRegion(String begin, String over) {
        List<RoadDataDtos> roadDataDtos = roadDataMapper.listRoadDataByTimeRegion(begin, over);
        roadDataDtos.forEach(roadData1 -> {
            roadData1.setDt(roadData1.getDt().substring(0, 10));
        });
        return Result.success(roadDataDtos);
    }

    /**
     * 查询全部道路污染累计事件数量
     *
     * @return
     */
    @GetMapping("selectRoadData")
    @ApiOperation("查询全部道路污染累计事件数量")
    public Result selectRoadData(Integer page, Integer pageSize) {
        List<RoadData> roadData = roadDataMapper.selectRoadData(page, pageSize);
        roadData.forEach(roadData1 -> {
            roadData1.setDt(roadData1.getDt().substring(0, 10));
        });
        Integer integer = roadDataMapper.selectCountRoadData();
        Map<String, Object> map = new HashMap<>();
        map.put("sum", integer);
        map.put("roadData", roadData);
        Result success = Result.success(map);
        return success;
    }


    //查询工地信息，根据经纬度和半径查询
    @GetMapping(value = "listBuildingSiteByPoints")
    @ApiOperation("查询工地信息，根据经纬度和半径从表中查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "longitude", value = "经度", paramType = "query", required = true, dataType = "double"),
            @ApiImplicitParam(name = "longitude", value = "经度", paramType = "query", required = true, dataType = "double"),
            @ApiImplicitParam(name = "radius", value = "半径", paramType = "query", required = true, dataType = "Integer")
    })
    public Result listBuildingSiteByPoints(double longitude, double latitude, Integer radius) {
        List<BuildingSite> list = new ArrayList<>();
        list = buildingSitrService.listBuildingSiteByPoints(longitude, latitude, radius);
        return Result.success(list);
    }

    /**
     * 查询全部工地信息
     *
     * @return
     */
    @GetMapping("selectBuildSite")
    @ApiOperation("查询全部工地信息")
    public Result selectBuildSite() {
        List<BuildingSite> buildingSites = buildingSiteMapper.selectBuildSite();
        Result success = Result.success(buildingSites);
        return success;
    }

}
