package com.hisense.monitormanage.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.hisense.monitormanage.dto.BuildingRecordsDto;
import com.hisense.monitormanage.dto.BuildingRecordsDtos;
import com.hisense.monitormanage.dto.EventDto;
import com.hisense.monitormanage.dto.RoadDataDto;
import com.hisense.monitormanage.entity.BuildingRecords;
import com.hisense.monitormanage.entity.Event;
import com.hisense.monitormanage.entity.RoadData;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @author admin
 * @version 1.0.0
 * @ClassName BuildingRecordsMapper.java
 * @Description TODO
 * @createTime 2022年05月17日 15:32:00
 */
@Repository
public interface BuildingRecordsMapper extends BaseMapper<BuildingRecords> {

    void batchSave(@Param("list") List<Map> list);

    void batchUpdate(@Param("list") List<Map> list);

    @Select("select * from (select bs.gdmc,bs.sgwz,bs.jd,bs.wd,bs.ssdq,bs.yjsg,bs.channel_code,bs.channel_name,b.* from t_building_site bs LEFT JOIN (select a.*,br.appid,br.noice,br.pm10,br.project_name FROM t_building_records br JOIN (select build_license,MAX(push_time) AS push_time  from t_building_records  GROUP BY build_license) a ON br.build_license = a.build_license AND br.push_time = a.push_time) b ON bs.gdmc = b.project_name ) c LEFT JOIN t_channel_picture cp ON c.channel_code = cp.channel_code ORDER BY c.pm10 DESC")
    List<BuildingRecordsDto> selectAllBuilding();

    List<BuildingRecordsDtos> selectWeekPmAndNoice(@Param("buildLicense") String buildLicense);

    List<BuildingRecordsDtos> selectMonthPmAndNoice(@Param("buildLicense") String buildLicense);

    List<BuildingRecordsDto> selectRaise(@Param("page") Integer page, @Param("pageSize") Integer pageSize, @Param("startTime") String startTime, @Param("endTime") String endTime);

    List<BuildingRecordsDto> selectNoiceEvent(@Param("page") Integer page, @Param("pageSize") Integer pageSize);

    List<EventDto> selectByAllEvent(@Param("page") Integer page, @Param("pageSize") Integer pageSize);

    @Select("select COUNT(*) from t_event te JOIN t_scene_event tse ON te.camera_name = tse.channel_name where te.track_event = 'START' and tse.scene_id = 1 ")
    Integer selectCountEvent();

    @Select("select COUNT(*) from (select br.pm10 from t_building_records  br JOIN t_building_site bs ON br.project_name = bs.gdmc where br.pm10 > 80) a ")
    Integer selectCountRaise();

    @Select("SELECT COUNT(*) FROM (SELECT br.noice,br.push_time FROM t_building_records  br JOIN t_building_site bs ON br.project_name = bs.gdmc WHERE (hour(br.push_time)>=22 or 6 > hour(br.push_time)) AND br.noice > 55 " +
            "UNION SELECT br.noice,br.push_time FROM t_building_records  br JOIN t_building_site bs ON br.project_name = bs.gdmc WHERE DATE_FORMAT(br.push_time,'%H')>=6 AND DATE_FORMAT(br.push_time,'%H')<=22 AND br.noice > 70) a")
    Integer selectCountNoice();

    @Select("select * from t_building_site bs JOIN (select a.*,br.appid,br.noice,br.pm10,br.project_name FROM t_building_records br JOIN  (select build_license,MAX(push_time) AS push_time  from t_building_records  GROUP BY build_license) a ON br.build_license = a.build_license AND br.push_time = a.push_time) b  on b.project_name = bs.gdmc where b.project_name like concat('%',#{projectName},'%') ORDER BY b.push_time DESC")
    List<BuildingRecordsDto> selectByProjectName(String projectName);

    @Select("select * from t_building_site bs LEFT JOIN (select a.*,br.appid,br.noice,br.pm10,br.project_name FROM t_building_records br JOIN (select build_license,MAX(push_time) AS push_time  from t_building_records  GROUP BY build_license) a ON br.build_license = a.build_license AND br.push_time = a.push_time) b ON b.project_name = bs.gdmc WHERE (bs.jd between #{jd} and #{jd1}) and (bs.wd between #{wd} and #{wd1})")
    List<BuildingRecordsDto> selectByJdWd(@Param("jd") Double jd,
                                          @Param("jd1") Double jd1,
                                          @Param("wd") Double wd,
                                          @Param("wd1") Double wd1);

    @Select("select * from t_building_records  br JOIN t_building_site bs ON br.project_name = bs.gdmc where TO_DAYS(push_time) = TO_DAYS(NOW()) AND br.pm10 > 80")
    List<BuildingRecordsDto> selectByDayRaise();

    @Select("SELECT a.* FROM (SELECT br.noice,br.push_time,br.project_name,bs.jd,bs.wd,bs.channel_name,bs.sgwz,bs.ssdq FROM t_building_records  br JOIN t_building_site bs ON br.project_name = bs.gdmc WHERE (hour(br.push_time)>=22 or 6 > hour(br.push_time)) AND br.noice > 55 AND TO_DAYS(br.push_time) = TO_DAYS(NOW()) UNION SELECT br.noice,br.push_time,br.project_name,bs.jd,bs.wd,bs.channel_name,bs.sgwz,bs.ssdq FROM t_building_records  br JOIN t_building_site bs ON br.project_name = bs.gdmc WHERE DATE_FORMAT(br.push_time,'%H')>=6 AND DATE_FORMAT(br.push_time,'%H') < 22 AND br.noice > 70 AND TO_DAYS(br.push_time) = TO_DAYS(NOW())) a")
    List<BuildingRecordsDto> selectByDayNoice();

    @Select("select * from t_road_data where pm10 > 150 AND dt = #{dt}")
    List<RoadData> selectByDayRoadData(String dt);

    @Select("select * from t_event te JOIN t_scene_event tse ON te.camera_name = tse.channel_name where te.track_event = 'START' AND tse.scene_id = 1 AND TO_DAYS(te.capture_time) = TO_DAYS(NOW())")
    List<EventDto> selectByEvent();

    List<BuildingRecordsDto> selectByWeekNoice();

    List<BuildingRecordsDto> selectByMonthNoice();

    List<BuildingRecordsDto> selectByWeekNaise();

    List<BuildingRecordsDto> selectByMonthNaise();

    List<EventDto> selectByWeekEvent();

    List<EventDto> selectByMonthEvent();

    List<BuildingRecordsDtos> selectMonthCurrentByBuilding();

    List<BuildingRecordsDtos> selectMonthCurrentByEvent();

    @Select("select rd.road_name,rd.`start`,rd.`end`,COUNT(*) number from t_road_data rd where pm10 > 150 GROUP BY rd.road_name,rd.`start`,rd.`end` ORDER BY number DESC")
    List<RoadDataDto> selectRoadDataCountEvent();

    @Select("select rd.road_name,rd.`start`,rd.`end`,COUNT(*) number from t_road_data rd where pm10 > 150 AND DATE_SUB(CURDATE(), INTERVAL 7 DAY) < DATE(rd.dt) GROUP BY rd.road_name,rd.`start`,rd.`end` ORDER BY number DESC")
    List<RoadDataDto> selectRoadDataWeekEvent();

    @Select("select rd.road_name,rd.`start`,rd.`end`,COUNT(*) number from t_road_data rd where pm10 > 150 AND DATE_SUB(CURDATE(), INTERVAL 30 DAY) < DATE(rd.dt) GROUP BY rd.road_name,rd.`start`,rd.`end` ORDER BY number DESC")
    List<RoadDataDto> selectRoadDataMonthEvent();

    List<BuildingRecordsDtos> selectMonthCurrentByRoadData();

}
