package com.hisense.monitormanage.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.hisense.monitormanage.dto.EventDto;
import com.hisense.monitormanage.dto.KeyAreaDto;
import com.hisense.monitormanage.dto.KeyAreaDtos;
import com.hisense.monitormanage.entity.KeyArea;
import com.hisense.monitormanage.entity.KeyAreaLocation;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface KeyAreaMapper extends BaseMapper<KeyArea> {

    List<KeyAreaDto> selectKeyAreaList(@Param("type1") String type1, @Param("type2") String type2, @Param("type3") String type3, @Param("timeId") String timeId);

    List<KeyAreaDto> selectKeyArea(@Param("type") String type, @Param("timeId") String timeId);

    @Select("SELECT * FROM t_key_area_location kal JOIN t_key_area ka ON kal.area_id = ka.area_id WHERE ka.area_name = #{areaName}")
    List<KeyAreaLocation> selectByAreaName(String areaName);

    @Select("select * from t_key_area where area_name like concat('%',#{areaName},'%')")
    List<KeyArea> selectByLikeName(String areaName);

    List<EventDto> selectByAreaEvent(@Param("page") Integer page, @Param("pageSize") Integer pageSize);

    @Select("SELECT COUNT(*) FROM t_event te JOIN t_scene_event tse ON te.camera_name = tse.channel_name WHERE te.track_event = 'START' AND tse.scene_id = 2")
    Integer selectCountAreaEvent();

    @Select("SELECT * FROM t_event te JOIN t_scene_event tse ON te.camera_name = tse.channel_name WHERE te.track_event = 'START' AND tse.scene_id = 2 AND TO_DAYS(te.capture_time) = TO_DAYS(NOW())")
    List<EventDto> selectByDayAreaEvent();

    List<EventDto> selectByNameAreaEvent(@Param("eventNewName") String eventNewName, @Param("page") Integer page, @Param("pageSize") Integer pageSize);

    @Select("SELECT COUNT(*) FROM t_event te JOIN t_scene_event tse ON te.camera_name = tse.channel_name WHERE te.track_event = 'START' AND tse.scene_id = 2 AND event_cn_name IN (${eventNewName})")
    Integer selectCountByNameAreaEvent(@Param("eventNewName") String eventNewName);

    List<KeyAreaDtos> selectMonthAreaEvent();

    @Select("SELECT te.event_cn_name,COUNT(*) AS number FROM t_event te JOIN t_scene_event tse ON te.camera_name = tse.channel_name WHERE te.track_event = 'START' AND tse.scene_id = 2 GROUP BY te.event_cn_name")
    List<KeyAreaDtos> selectEventType();

    List<KeyAreaDtos> selectLocalWeekArea(@Param("areaName") String areaName);

    List<KeyAreaDtos> selectLocalMonthArea(@Param("areaName") String areaName);

    @Select("select * from t_key_area ka LEFT JOIN t_channel_picture cp ON ka.channel_code = cp.channel_code where ka.area_name = #{areaName} ")
    List<KeyAreaDto> selectByAreaImage(@Param("areaName") String areaName);

}
