package com.hisense.monitormanage.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.hisense.monitormanage.dto.BuildingRecordsDto;
import com.hisense.monitormanage.entity.BuildingRecords;
import com.hisense.monitormanage.mapper.BuildingRecordsMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.FileOutputStream;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author admin
 * @version 1.0.0
 * @ClassName BuildingRecordsService.java
 * @Description TODO
 * @createTime 2022年05月17日 15:33:00
 */
@Service
@Slf4j
public class BuildingRecordsService {

    @Autowired
    private BuildingRecordsMapper buildingRecordsMapper;

    @Autowired
    private RestTemplate restTemplate;
    @Value("${monitorData.url}")
    private String monitorDataUrl;

    public List<Map> getRecords() {
        String url = monitorDataUrl;
        Map<String, String> map = new HashMap<>();
        map.put("appid", "");
        map.put("timestamp", "");
        map.put("sign", "");
        map.put("data", "");
        ResponseEntity<JSONObject> responseEntity;
        List<Map> maps = new ArrayList<>();
        try {
            responseEntity = restTemplate.postForEntity(url, map, JSONObject.class);
            if (responseEntity.getStatusCodeValue() == 200) {
                JSONObject result = responseEntity.getBody();
                if (result.getIntValue("code") == 0) {
                    JSONArray jsonArray = result.getJSONArray("data");
                    if (jsonArray != null) {
                        maps = JSONObject.parseArray(JSONArray.toJSONString(jsonArray), Map.class);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("[BuildingRecordsService-getRecords] Exception:" + e.getMessage());
        }
        return maps;
    }

    @Scheduled(cron = "0 0 */8 * * ?")
    public boolean getAndSaveRecords() {
        List<Map> maps = this.getRecords();
        boolean result = false;
        if (maps.size() > 0) {
            List<List<Map>> lists = Lists.partition(maps, 200);
            try {
                lists.forEach(l -> buildingRecordsMapper.batchSave(l));
                result = true;
            } catch (Exception e) {
                e.printStackTrace();
                log.error("[BuildingRecordsService-getAndSaveRecords] Exception:" + e.getMessage());
            }
        }
        return result;
    }


    public boolean getUpdateRecords() {
        List<Map> maps = this.getRecords();
        boolean result = false;
        if (maps.size() > 0) {
            List<List<Map>> lists = Lists.partition(maps, 200);
            try {
                lists.forEach(l -> buildingRecordsMapper.batchUpdate(l));
                result = true;
            } catch (Exception e) {
                e.printStackTrace();
                log.error("[BuildingRecordsService-getUpdateRecords] Exception:" + e.getMessage());
            }
        }
        return result;
    }

    public List<BuildingRecords> listRecords(String pushTime) {
        QueryWrapper<BuildingRecords> wrapper = new QueryWrapper<>();
        wrapper.like("push_time", pushTime);
        List<BuildingRecords> list = new ArrayList<>();
        list = buildingRecordsMapper.selectList(wrapper);
        return list;
    }

    //导出excel表格
    public void testHSSF(String startTime, String endTime) throws Exception {
        // 1. 创建一个工作簿
        HSSFWorkbook wb = new HSSFWorkbook();
        // 2. 创建一个工作表
        HSSFSheet sheet = wb.createSheet();
        // 3. 获取数据库数据
        List<BuildingRecordsDto> buildingRecordsDtos = buildingRecordsMapper.selectRaise(null, null, startTime, endTime);
        for (int i = 0; i < buildingRecordsDtos.size(); i++) {
            // 4. 创建行
            HSSFRow row = sheet.createRow(i);
            // 5. 使用反射获取实体的元素的总量
            Class<? extends BuildingRecordsDto> aClass = buildingRecordsDtos.get(i).getClass();
            Field[] fields = aClass.getDeclaredFields();
            // 6. 行内单元格赋值
            for (int j = 0; j < fields.length; j++) {
                Field f = fields[j];
                f.setAccessible(true);
                HSSFCell cell = row.createCell(j);
                cell.setCellValue(String.valueOf(f.get(buildingRecordsDtos.get(i))));
            }
        }
        //  7.写文件
        FileOutputStream fos = new FileOutputStream("D:/" + System.currentTimeMillis() + ".xls");
        // 8.关闭流
        wb.write(fos);
        fos.close();
    }
}
