package com.hisense.monitormanage.service;

import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.hisense.monitormanage.dto.ChannelLabelDto;
import com.hisense.monitormanage.entity.*;
import com.hisense.monitormanage.mapper.*;
import lombok.extern.log4j.Log4j2;
import org.bytedeco.javacv.FFmpegFrameGrabber;
import org.bytedeco.javacv.Frame;
import org.bytedeco.javacv.Java2DFrameConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.Base64Utils;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StringUtils;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;

import javax.annotation.PostConstruct;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.List;


@Service
@Log4j2
public class MonitorService {

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private CameraMapper cameraMapper;

    @Autowired
    private ScenicMapper scenicMapper;

    @Autowired
    private CameraScenicMapper cameraScenicMapper;

    @Autowired
    private CameraChannelMapper cameraChannelMapper;

    @Autowired
    private CameraOrgenizationMapper cameraOrgenMapper;

    @Autowired
    private ChannelPictureMapper channelPictureMapper;

    @Autowired
    private BoothMapper boothMapper;

    @Autowired
    private PeopleRecognizeMapper peopleRecognizeMapper;

    @Autowired
    private TestCodeMapper testCodeMapper;

    @Autowired
    private WCameraMapper wCameraMapper;

    @Autowired
    private BuildingSiteMapper buildingSiteMapper;

    @Value("${hwx.file.work-path}")
    private String fileDir;
    @Value("${hwx.file.pic-host}")
    private String picHost;

    @Value("iOgQotfgfyLvhj6WgfDTpq7F")
    private String key;

//    final private String userName = "chengshiyunnao";
//    final private String password = "QDyjj@2021";

    final private String userName = "ynszdz";
    final private String password = "Admin@123";
    final private String monitorDomain = "http://10.132.191.3:8314";

    private String token;

    static int keepaliveCount = 16;

    @PostConstruct
    public void init() {

        String loginUrl = monitorDomain + "/videoService/accounts/authorize";
        HashMap<String, String> loginParam = new HashMap<>();
        loginParam.put("userName", userName);
        loginParam.put("clientType", "winpc");

        //第一次登录交换
        HashMap reponse = null;
        try {
            ResponseEntity<HashMap> loginResEntity = restTemplate.postForEntity(loginUrl, loginParam, HashMap.class, new HashMap<>());
            reponse = loginResEntity.getBody();
        } catch (HttpClientErrorException e) {
            if (e.getStatusCode() == HttpStatus.UNAUTHORIZED) {
                String responseString = e.getResponseBodyAsString();
                reponse = JSONObject.parseObject(responseString, HashMap.class);
            } else {
                reponse = null;
            }
        } catch (Exception exception) {
            reponse = null;
        }

        if (reponse == null) {
            log.error("[monitor-capture]:" + "监控系统登录握手失败");
            return;
        }

        //登录
        loginParam.putAll(reponse);
        String sigin = SecureUtil.md5(password);
        sigin = SecureUtil.md5(userName + sigin);
        sigin = SecureUtil.md5(sigin);
        sigin = SecureUtil.md5(userName + ":" + loginParam.get("realm") + ":" + sigin);
        sigin = SecureUtil.md5(sigin + ":" + loginParam.get("randomKey"));
        loginParam.put("signature", sigin);

        reponse = restTemplate.postForEntity(loginUrl, loginParam, HashMap.class, new HashMap<>()).getBody();
        if (reponse != null) {
            String token = (String) reponse.get("token");
            if (token == null || StringUtils.isEmpty(token)) {
                log.error("[monitor-capture]:" + "监控系统登录失败");
                return;
            }
            this.token = token;
            Integer duration = (Integer) reponse.get("duration");
            if (duration == null || duration < 20) {
                log.error("[monitor-capture]:" + "监控系统失效时间异常" + duration);
                return;
            }

            duration = duration - 20;

            MonitorService thatMonitorService = this;

            //会话保活
            Timer timer = new Timer("monitor-capture-keepalive");
            timer.scheduleAtFixedRate(new TimerTask() {

                @Override
                public void run() {

                    try {
                        String keepaliveUrl = monitorDomain + "/videoService/accounts/token/keepalive";
                        HttpHeaders headers = new HttpHeaders();
                        headers.add("X-Subject-Token", token);

                        HashMap<Object, Object> keepaliveParam = new HashMap<>();
                        keepaliveParam.put("token", token);
                        HttpEntity<Object> httpEntity = new HttpEntity<>(keepaliveParam, headers);
                        restTemplate.put(keepaliveUrl, httpEntity);

                        if (keepaliveCount++ > 12) {//大概36分钟输出一次日志
                            log.info("[monitor-capture]: keepalive success");
                            keepaliveCount = 0;
                        }

                    } catch (Exception e) {
                        log.info("[monitor-capture]: keepalive faild, restart.....");
                        e.printStackTrace();

                        //出错后重新登录
                        timer.cancel();
                        thatMonitorService.start();
                    }
                }
            }, 0, duration * 1000);
        }
    }

    @Scheduled(cron = "0 0 8 * * ?")
    public void start() {

        try {

            log.info("[monitor-capture]: start capture1");

            List<Camera> cameras = cameraMapper.selectList(null);
            cameras.forEach(camera -> {
                String cameraCode = camera.getCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(cameraCode);
                    String fileName = cameraCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;
                    camera.setPic(picUrl);
                    camera.setCaptureTime(new Date());
                    cameraMapper.updateById(camera);
                } catch (Exception e) {

                    log.info("[monitor-capture] " + cameraCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture1");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 工地农贸市场抓图
     */
    @Scheduled(cron = "0 0 8 * * ?")
    public void construction() {

        try {

            log.info("[monitor-capture]: start capture1");

            List<ChannelLabelDto> labels = cameraChannelMapper.selectLabelAndChannel("1009,1008");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture1");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 海水浴场景区抓图
     */
    @Scheduled(cron = "0 0 8/1 * * ?")
    public void scenicSpotAndBathing() {

        try {
            log.info("[monitor-capture]: start capture");
            List<ChannelLabelDto> labels = cameraChannelMapper.selectLabelAndChannel("1006,1004");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 窗口单位广场街区抓图
     */
    @Scheduled(cron = "0 0 8/1 * * ?")
    public void scenicSpotAndBathing1() {

        try {
            log.info("[monitor-capture]: start capture");
            List<ChannelLabelDto> labels = cameraChannelMapper.selectLabelAndChannel("1001,1005,1007");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    //@Scheduled(cron = "0 30 8 * * ?")
    /*public void buildingImage() {
        try {
            List<BuildingSite> buildingSites = buildingSiteMapper.selectByList();

            buildingSites.forEach(building -> {
                String channelCode = building.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));
                    String picUrl = picHost + "/images/" + fileName;
                    ChannelPicture channelPicture = new ChannelPicture();
                    channelPicture.setChannelCode(channelCode);
                    channelPicture.setPicUrl(picUrl);
                    channelPicture.setInsertTime(new Date());
                    channelPictureMapper.insert(channelPicture);
                }catch (Exception e){
                    log.info( "[monitor-capture] " +channelCode + " 截图图片失败 " + e.getMessage());
                }
            });
            log.info("[monitor-capture]: end capture");
        }catch (Exception exception){
            exception.printStackTrace();
        }
    }*/


    /**
     * 所有摄像头抓图 sql持续更新
     * TODO 还没有更新完
     */
    @Scheduled(cron = "0 0 6 * * ?")
    public void CameraScreenshot() {
        try {
            log.info("[monitor-capture]: start capture");

            List<CameraChannel> cameraChannels = cameraChannelMapper.selectCameraChannel();

            cameraChannels.forEach(channel -> {
                String channelCode = channel.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));
                    String picUrl = picHost + "/images/" + fileName;
                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);
                } catch (Exception e) {
                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                    TestCode testCode = new TestCode();
                    testCode.setChannelCode(channelCode);
                    testCodeMapper.insert(testCode);
                }
            });
            log.info("[monitor-capture]: end capture");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 海水浴场人群图片算法识别
     */
    public void bathingBeach() {
        try {

            log.info("[monitor-capture]: start capture");

            List<ChannelLabelDto> labels = cameraChannelMapper.selectLabelAndChannel("1006");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));
                    String picUrl = picHost + "/images/" + fileName;
                    //人流识别
                    String personNumUrl = "http://15.72.184.10:8003/cshi_rpc/v1.0/person";
                    Map personImage = new HashMap<>();
                    ByteArrayOutputStream stream = new ByteArrayOutputStream();
                    ImageIO.write(bufferedImage, "png", stream);
                    String imgBase64 = Base64Utils.encodeToString(stream.toByteArray());
                    stream.flush();
                    stream.close();
                    personImage.put("img_base64", imgBase64);
                    ResponseEntity<HashMap> responseEntity = restTemplate.postForEntity(personNumUrl, personImage, HashMap.class);
                    HashMap body = responseEntity.getBody();
                    if (body != null && (Integer) body.get("code") == 200) {
                        Map data = (Map) body.get("data");
                        log.info(picUrl);
                        log.info("人流识别算法" + data);
                    }
                } catch (Exception e) {
                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });
            log.info("[monitor-capture]: end capture");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 安保区域人群图片算法识别并保存到数据库
     */
    @Scheduled(cron = "0 0 8/2 * * ?")
    public void peopleRecognize() {
        try {
            log.info("[monitor-capture]: start capture");

            List<ChannelLabelDto> labels = cameraChannelMapper.selectLabelAndChannel("1042");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));
                    String picUrl = picHost + "/images/" + fileName;
                    //人流识别
                    String personNumUrl = "http://15.72.184.10:8003/cshi_rpc/v1.0/person";
                    Map personImage = new HashMap<>();
                    ByteArrayOutputStream stream = new ByteArrayOutputStream();
                    ImageIO.write(bufferedImage, "png", stream);
                    String imgBase64 = Base64Utils.encodeToString(stream.toByteArray());
                    stream.flush();
                    stream.close();
                    personImage.put("img_base64", imgBase64);
                    ResponseEntity<HashMap> responseEntity = restTemplate.postForEntity(personNumUrl, personImage, HashMap.class);
                    HashMap body = responseEntity.getBody();
                    if (body != null && (Integer) body.get("code") == 200) {
                        Map data = (Map) body.get("data");
                        Integer personNum = (Integer) data.get("person_num");
                        PeopleRecognize peopleRecognize = new PeopleRecognize();
                        peopleRecognize.setChannelCode(channelCode);
                        peopleRecognize.setChannelName(label.getChannelName());
                        peopleRecognize.setPicUrl(picUrl);
                        peopleRecognize.setPersonNum(personNum);
                        if (channelCode == null) {
                            peopleRecognizeMapper.insert(peopleRecognize);
                        } else {
                            QueryWrapper<PeopleRecognize> queryWrapper = new QueryWrapper<>();
                            queryWrapper.eq("channel_code", channelCode);
                            peopleRecognize.setInsertTime(new Date());
                            peopleRecognizeMapper.update(peopleRecognize, queryWrapper);
                        }
                    }
                } catch (Exception e) {
                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });
            log.info("[monitor-capture]: end capture");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 景区人流量识别
     */
    @Scheduled(cron = "0 0 9 * * ?")
    public void startScenic() {
        try {

            log.info("[monitor-capture]: startScenic capture");

            List<Camera> cameras = scenicMapper.selectCamerasInScenic();
            cameras.forEach(camera -> {
                String cameraCode = camera.getCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(cameraCode);
                    String fileName = cameraCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;
                    camera.setPic(picUrl);
                    camera.setCaptureTime(new Date());
                    cameraMapper.updateById(camera);
                    //人流识别
                    String personNumUrl = "http://15.72.184.10:8003/cshi_rpc/v1.0/person";
                    Map personImage = new HashMap<>();
                    ByteArrayOutputStream stream = new ByteArrayOutputStream();
                    ImageIO.write(bufferedImage, "png", stream);
                    String imgBase64 = Base64Utils.encodeToString(stream.toByteArray());
                    stream.flush();
                    stream.close();
                    personImage.put("img_base64", imgBase64);
                    ResponseEntity<HashMap> responseEntity = restTemplate.postForEntity(personNumUrl, personImage, HashMap.class);
                    HashMap body = responseEntity.getBody();
                    if (body != null && (Integer) body.get("code") == 200) {
                        Map data = (Map) body.get("data");
                        Integer personNum = (Integer) data.get("person_num");
                        CameraScenic cameraScenic = new CameraScenic();
                        cameraScenic.setCameraId(cameraCode);
                        cameraScenic.setPersonNum(personNum);

                        CameraScenic cs = cameraScenicMapper.selectById(cameraCode);
                        if (cs == null) {
                            cameraScenicMapper.insert(cameraScenic);
                        } else {
                            cameraScenicMapper.updateById(cameraScenic);
                        }


                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + cameraCode + " 人流识别失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: startScenic end capture");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    private BufferedImage capturecreenshot(String code) throws IOException {
        String url = monitorDomain + "/videoService/realmonitor/uri?scheme=HLS&channelId=" + code;

        HttpHeaders headers = new HttpHeaders();
        headers.add("X-Subject-Token", token);

        ResponseEntity<HashMap> forEntity = restTemplate.exchange(url, HttpMethod.GET, new HttpEntity(headers), HashMap.class);

        HashMap body = forEntity.getBody();

        String file = (String) body.get("url");

        log.debug("开始抓流：" + file);

        FFmpegFrameGrabber grabber = FFmpegFrameGrabber.createDefault(file);
        try {

            grabber.setOption("stimoout", "10*1000*1000");
            grabber.setOption("timeout", "10*1000*1000");

            grabber.start();

            log.debug("grabber启动成功");

            int frameIndex = 60;
            Java2DFrameConverter converter = new Java2DFrameConverter();
            while (true) {
                Frame frame = grabber.grabImage();
                if (frame != null) {
                    log.debug("grabber 跳帧：" + frameIndex);
                    if (frameIndex-- > 0) continue;
                    BufferedImage bufferedImage = converter.getBufferedImage(frame);
                    return bufferedImage;
                }
            }
        } catch (Exception e) {
            throw e;
        } finally {
            grabber.stop();
            grabber.release();
        }
    }

    /**
     * 获取路径返回给前端
     *
     * @param code
     * @return
     * @throws IOException
     */
    public String fileCode(String code) throws IOException {
        String url = monitorDomain + "/videoService/realmonitor/uri?scheme=HLS&channelId=" + code;

        HttpHeaders headers = new HttpHeaders();
        headers.add("X-Subject-Token", token);

        System.out.println(token);

        ResponseEntity<HashMap> forEntity = restTemplate.exchange(url, HttpMethod.GET, new HttpEntity(headers), HashMap.class);

        HashMap body = forEntity.getBody();

        String file = (String) body.get("url");


        return file;

    }

    static public List cameras = new LinkedList<Map>();

    /**
     * 调用接口获取摄像头和组织
     *
     * @param parentId
     * @param path
     * @throws IOException
     * @throws InterruptedException
     */
    public void videoService(String parentId, String path) throws IOException, InterruptedException {

        String url = monitorDomain + "/videoService/devicesManager/deviceTree?nodeType=1&typeCode=01;0;ALL;ALL&id=" + parentId;

        HttpHeaders headers = new HttpHeaders();

        headers.add("X-Subject-Token", token);

        ResponseEntity<List> forEntity = null;
        for (int i = 0; i < 10; i++) {
            try {
                forEntity = restTemplate.exchange(url, HttpMethod.GET, new HttpEntity(headers), List.class);
                break;
            } catch (Exception e) {
                log.error("重试次数：{} url:{}", i, url);
                log.error(e);
                Thread.sleep(5 * 1000);
            }
        }

        if (forEntity == null) return;
        List<Map> results = forEntity.getBody();

//        log.debug("response {}", JSONObject.toJSONString(results));
        String sep = "->";

        for (Map result : results) {
            Boolean isParent = (Boolean) result.get("isParent");

            if (isParent) {
                CameraOrganization cameraOrganization = JSONObject.parseObject(JSONObject.toJSONString(result), CameraOrganization.class);

                cameraOrgenMapper.insert(cameraOrganization);

                videoService((String) result.get("id"), path + sep + result.get("name"));
            } else {
                if (result.get("channelId") != null) {
                    CameraChannel cameraChannel = JSONObject.parseObject(JSONObject.toJSONString(result), CameraChannel.class);

                    String id = (String) result.get("orgCode");

                    if (path != null && path.startsWith(sep)) {
                        path = path.substring(sep.length());
                    }
                    cameraChannel.setNodeName(path);

                    cameraChannel.setParentId(id);

                    cameraChannelMapper.insert(cameraChannel);

                }

            }
        }
    }

    //查询视频点播巡检结果
    @Scheduled(cron = "0 5 6 * * ?")
    public Result listChannelPlayStates() {
        //获取当前时间和前一天的UTC时间
        Map<String, Object> condition = new HashMap<>();
        ZonedDateTime endUTC = ZonedDateTime.now(ZoneOffset.UTC);
        ZonedDateTime startUTC = endUTC.minusDays(90);
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyyMMdd'T'HHmmss'Z'");
        condition.put("checkStatus", 0);
        condition.put("startTime", dateTimeFormatter.format(startUTC));
        condition.put("endTime", dateTimeFormatter.format(endUTC));
        //组装查询条件，经过验证pageSize设置过大也没效果，最大512
        Map<String, Object> map = new HashMap<>();
        map.put("condition", condition);
        map.put("page", 1);
        map.put("pageSize", 500);//最多一次512条
        //查询路径和请求头信息
        String url = monitorDomain + "/nms/api/channel/play/list";
        HttpHeaders headers = new HttpHeaders();
        headers.add("X-Subject-Token", token);

        //查询数据的页数
        int pageCount = 0;
        //查询结果
        List<Map> list = new ArrayList<>();

        HttpEntity<Map<String, Object>> request = new HttpEntity<>(map, headers);
        ResponseEntity<JSONObject> responseEntity = restTemplate.exchange(url, HttpMethod.POST, request, JSONObject.class);
        JSONObject entityBody = responseEntity.getBody();
        if (entityBody.get("i18n") != null && entityBody.get("i18n").toString() == "error") {
            log.info("{}程序异常{}", entityBody.get("message").toString());
        }
        List<Map> results = (List<Map>) entityBody.get("results");
        if (!results.isEmpty()) {
            list.addAll(results);
            pageCount = entityBody.getIntValue("totalCount") / 500 + 1;
        }
        for (int i = 2; i <= pageCount; i++) {
            map.put("page", i);
            HttpEntity<Map<String, Object>> request2 = new HttpEntity<>(map, headers);
            ResponseEntity<JSONObject> responseEntity2 = restTemplate.exchange(url, HttpMethod.POST, request2, JSONObject.class);
            JSONObject entityBody2 = responseEntity2.getBody();
            List<Map> results2 = (List<Map>) entityBody2.get("results");
            log.info("循环次数：" + i);
            if (!results2.isEmpty()) {
                list.addAll(results);
            }
        }
        //通过set对channel_code去重
        Set<String> set = new HashSet<>();

        for (Map m : list) {
            set.add(m.get("channelCode").toString());
        }
        List<String> lists = new ArrayList<>(set.size());
        lists.addAll(set);
        //根据channel_code对t_camera_channel表的status字段进行更新（0）
        List<List<String>> listMap = Lists.partition(lists, 100);
        for (List<String> ll : listMap) {
            cameraChannelMapper.updateCameraStatus(ll);
        }
        log.info("{} 查询视频点播巡检成功，发现异常数量：{}", LocalDateTime.now(), set.size());
        return Result.success();
    }

    /**
     * 特定摊点8:20抓图
     */
    @Scheduled(cron = "0 20 8 * * ? ")
    public void boothEight() {
        try {
            log.info("[monitor-capture]: start capture1");

            List<Booth> labels = boothMapper.selectBooth("8:20");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture1");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 特定摊点8:50抓图
     */
    @Scheduled(cron = "0 50 8 * * ? ")
    public void boothEights() {
        try {
            log.info("[monitor-capture]: start capture1");

            List<Booth> labels = boothMapper.selectBooth("8:50");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture1");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 特定摊点9:50抓图
     */
    @Scheduled(cron = "0 50 9 * * ? ")
    public void boothNine() {
        try {
            log.info("[monitor-capture]: start capture1");

            List<Booth> labels = boothMapper.selectBooth("9:50");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture1");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 特定摊点9:20抓图
     */
    @Scheduled(cron = "0 20 9 * * ? ")
    public void boothNines() {
        try {
            log.info("[monitor-capture]: start capture1");

            List<Booth> labels = boothMapper.selectBooth("9:20");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture1");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 特定摊点11:20抓图
     */
    @Scheduled(cron = "0 20 11 * * ? ")
    public void boothEleven() {
        try {
            log.info("[monitor-capture]: start capture1");

            List<Booth> labels = boothMapper.selectBooth("11:20");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture1");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 特定摊点12:20抓图
     */
    @Scheduled(cron = "0 20 12 * * ? ")
    public void boothTwelve() {
        try {
            log.info("[monitor-capture]: start capture1");

            List<Booth> labels = boothMapper.selectBooth("12:20");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture1");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 特定摊点19:20抓图
     */
    @Scheduled(cron = "0 20 19 * * ? ")
    public void boothNineteen() {
        try {
            log.info("[monitor-capture]: start capture1");

            List<Booth> labels = boothMapper.selectBooth("19:20");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture1");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 特定摊点20:20抓图
     */
    @Scheduled(cron = "0 20 20 * * ? ")
    public void boothTwenty() {
        try {
            log.info("[monitor-capture]: start capture1");

            List<Booth> labels = boothMapper.selectBooth("20:20");
            labels.forEach(label -> {
                String channelCode = label.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));

                    String picUrl = picHost + "/images/" + fileName;

                    ChannelPicture picture = new ChannelPicture();
                    picture.setChannelCode(channelCode);
                    picture.setPicUrl(picUrl);
                    /*picture.setInsertTime(new Date());
                    channelPictureMapper.insert(picture);*/
                    if (channelCode == null) {
                        channelPictureMapper.insert(picture);
                    } else {
                        QueryWrapper<ChannelPicture> queryWrapper = new QueryWrapper<>();
                        queryWrapper.eq("channel_code", channelCode);
                        picture.setInsertTime(new Date());
                        channelPictureMapper.update(picture, queryWrapper);
                    }
                } catch (Exception e) {

                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });

            log.info("[monitor-capture]: end capture1");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * 积水点抓图
     */
    @Scheduled(cron = "0 0 8 * * ?")
    public void WaterPointService() {
        try {
            log.info("[monitor-capture]: start capture");
            QueryWrapper<WCamera> queryWrapper = new QueryWrapper<>();
            List<WCamera> wCameras = wCameraMapper.selectList(queryWrapper);
            wCameras.forEach(camera -> {
                String channelCode = camera.getChannelCode();
                try {
                    BufferedImage bufferedImage = capturecreenshot(channelCode);
                    String fileName = channelCode + ".jpg";
                    ImageIO.write(bufferedImage, "jpg", new File(fileDir + "/images/" + fileName));
                    String picUrl = picHost + "/images/" + fileName;
                    WCamera wCamera = new WCamera();
                    wCamera.setVideoCapture(picUrl);
                    QueryWrapper<WCamera> queryWrapper1 = new QueryWrapper<>();
                    queryWrapper1.eq("channel_code", channelCode);
                    wCameraMapper.update(wCamera, queryWrapper1);
                } catch (Exception e) {
                    log.info("[monitor-capture] " + channelCode + " 截图图片失败 " + e.getMessage());
                }
            });
            log.info("[monitor-capture]: end capture");
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }
}




