package com.hisense.monitormanage.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.hisense.monitormanage.dto.PassengerFlowDto;
import com.hisense.monitormanage.entity.PassengerFlow;
import com.hisense.monitormanage.mapper.PassengerFlowMapper;
import com.hisense.monitormanage.utils.LongLatUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.DigestUtils;
import org.springframework.web.client.RestTemplate;

import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * @author admin
 * @version 1.0.0
 * @ClassName PassengerFlowService.java
 * @Description 实时客流service
 * @createTime 2022年05月10日 14:45:00
 */
@Service
@Log4j2
public class PassengerFlowService {
    @Autowired
    private PassengerFlowMapper passengerFlowMapper;
    @Autowired
    private RestTemplate restTemplate;

    //实时客流相关参数
    @Value("202204141052331249fcc8-046")
    private String APIKEY;
    @Value("apitest")
    private String appId;
    @Value("7eb78b134e0310904d46039ccea20c32")
    private String appSecret;

    @Value("${scheduled.enable}")
    private Boolean enable;

    /**
     * 最新的实时客流
     *
     * @param
     * @return List
     */
    //{"APIKEY":"202204141052331249fcc8-046","ParamJson":{"data":"{\"appId\":\"apitest\",\"mac\":\"f51093001255129b88deba6c5045ee26\",\"timeStamp\":\"20220510152759\",\"hourId\":\"2022051013\",\"timeId\":\"202205101300\"}"}}
    public List<Map> passengerFlow() {
        Map<String, Object> map = new HashMap<>();
        DateTimeFormatter dateTimeFormatter1 = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");

        LocalDateTime dateNow = LocalDateTime.now();
        String timeStamp = dateNow.format(dateTimeFormatter1);//yyyyMMddHHmmss
        String[] tt = this.newMinusMinutesStr(dateNow);

        String url = "http://15.72.158.72:8081/getway/api/Proxy/HandleByKey/1249fcc8-0466-4897-87b4-d2111a9baf4f";
        map.put("APIKEY", APIKEY);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("appId", appId);
        //安全认证 MD5(appId+”-”+timeStamp+”-”+ appSecret)
        String code = appId + "-" + timeStamp + "-" + appSecret;
        String md5Str = DigestUtils.md5DigestAsHex(code.getBytes(StandardCharsets.UTF_8));
        jsonObject.put("mac", md5Str);
        jsonObject.put("timeStamp", timeStamp);
        jsonObject.put("hourId", tt[1]);
        jsonObject.put("timeId", tt[0]);
        JSONObject para = new JSONObject();
        para.put("data", JSONObject.toJSONString(jsonObject));
        map.put("ParamJson", para);

        System.out.println("multimap->" + map.toString());
        log.info("[passengerFlow] ->", map.toString());
        ResponseEntity<String> responseEntity;
        List<Map> list = new ArrayList<>();
        try {
            responseEntity = restTemplate.postForEntity(url, map, String.class);
            if (responseEntity.getStatusCodeValue() == 200) {
                String result = responseEntity.getBody();
                JSONObject jsonResult = JSONObject.parseObject(result);
                System.out.println(jsonResult.keySet().toString());
                if (jsonResult.getIntValue("code") == 0) {
                    JSONArray jsonArray = jsonResult.getJSONArray("result");
                    list = JSONObject.parseArray(JSONObject.toJSONString(jsonArray), Map.class);
                }
            }
        } catch (Exception e) {
            log.error("[passengerFlow] Exception:" + e.getMessage());
        }
        return list;
    }

    //从表中获取最新的实时客流
    public List<PassengerFlow> listPassengerFlow(String timeId) {
        List<PassengerFlow> lists = new ArrayList<>();
        long longTimeId = Long.valueOf(timeId);
        if (longTimeId == 1970000000) {
            PassengerFlow passengerFlow = passengerFlowMapper.getByMaxId();
            timeId = passengerFlow.getTimeId();
        }
        QueryWrapper<PassengerFlow> wrapper = new QueryWrapper<>();
        wrapper.eq("time_id", timeId);
        lists = passengerFlowMapper.selectList(wrapper);

        return lists;
    }

    //从表中获取获取最新的实时客流，并筛选出all_nums>=5000,并desc排序
    public List<PassengerFlow> listPassengerFlowByAllNums() {
        List<PassengerFlow> lists = new ArrayList<>();
        String timeId = passengerFlowMapper.getByMaxId().getTimeId();
        QueryWrapper<PassengerFlow> wrapper = new QueryWrapper<>();
        wrapper.eq("time_id", timeId).ge("all_nums", 5000).orderByDesc("all_nums");
        lists = passengerFlowMapper.selectList(wrapper);
        return lists;

    }

    @Scheduled(cron = "0 31 10-22/1 * * ? ")
    //提供的接口能查到当前时间2个小时之前的数据，每隔15分钟一次，我们只取整点从早8点到晚8点的整点数据
    //10:31 到晚上10：31，每小时的31分执行一次
    public void passengerFlowSchedule() {
        if (enable) {
            Map<String, Object> map = new HashMap<>();
            DateTimeFormatter dateTimeFormatter1 = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");

            LocalDateTime dateNow = LocalDateTime.now();
            String timeStamp = dateNow.format(dateTimeFormatter1);//yyyyMMddHHmmss
            String[] tt = this.minusMinutesStr(dateNow, 120);

            String url = "http://15.72.158.72:8081/getway/api/Proxy/HandleByKey/1249fcc8-0466-4897-87b4-d2111a9baf4f";
            map.put("APIKEY", APIKEY);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("appId", appId);
            //安全认证 MD5(appId+”-”+timeStamp+”-”+ appSecret)
            String code = appId + "-" + timeStamp + "-" + appSecret;
            String md5Str = DigestUtils.md5DigestAsHex(code.getBytes(StandardCharsets.UTF_8));
            jsonObject.put("mac", md5Str);
            jsonObject.put("timeStamp", timeStamp);
            jsonObject.put("hourId", tt[1]);
            jsonObject.put("timeId", tt[0]);
            JSONObject para = new JSONObject();
            para.put("data", JSONObject.toJSONString(jsonObject));
            map.put("ParamJson", para);

            System.out.println("multimap->" + map.toString());
            ResponseEntity<String> responseEntity;
            List<Map> list = new ArrayList<>();
            try {
                responseEntity = restTemplate.postForEntity(url, map, String.class);
                if (responseEntity.getStatusCodeValue() == 200) {
                    String result = responseEntity.getBody();
                    JSONObject jsonResult = JSONObject.parseObject(result);
                    System.out.println(jsonResult.keySet().toString());
                    if (jsonResult.getIntValue("code") == 0) {
                        JSONArray jsonArray = jsonResult.getJSONArray("result");
                        list = JSONObject.parseArray(JSONObject.toJSONString(jsonArray), Map.class);
                        List<List<Map>> lists = Lists.partition(list, 200);
                        lists.forEach(ll -> passengerFlowMapper.batchSave(ll));
                    }
                }
            } catch (Exception e) {
                log.error("[passengerFlowSchedule] Exception:" + e.getMessage());
            }
        }
    }

    //保存实时客流
    @Transactional(rollbackFor = Exception.class)
    public boolean savePassengerFlow(List<Map> maps) {
        List<List<Map>> lists = Lists.partition(maps, 200);
        try {
            lists.forEach(list -> passengerFlowMapper.batchSave(list));
            return true;
        } catch (Exception e) {
            log.error("[savePassengerFlow] Exception:" + e.getMessage());
            return false;
        }
    }

    //根据半径，一个经纬度和timeId查询游客总数
    public JSONObject passengerNums(double longitude, double latitude, Integer radius, String timeId) {
        int allCount = 0;
        int localCount = 0;
        int nonLocalCount = 0;
        JSONObject result = new JSONObject();

        try {
            double[] around = LongLatUtil.getAround(longitude, latitude, radius);
            List<PassengerFlowDto> list = passengerFlowMapper.selectPassenger(around[0], around[2], around[1], around[3], timeId);
            for (PassengerFlowDto p : list) {

                allCount += Integer.parseInt(p.getAllNums());
                localCount += Integer.parseInt(p.getLocalNums());
                nonLocalCount += Integer.parseInt(p.getNonlocalNums());
            }
            result.put("local_nums", localCount);
            result.put("nonlocal_nums", nonLocalCount);
            result.put("all_nums", allCount);
        } catch (Exception e) {
            log.error("[passengerNums] Exception:" + e.getMessage());
            return result;
        }
        return result;
    }

    //当前时间减去固定的分钟数，然后分钟取30
    private String[] minusMinutesStr(LocalDateTime datetime, long minutes) {
        String t1, t2;//t1 yyyyMMddHHmm  t2 yyyyMMddHH
        LocalDateTime minus = datetime.minus(minutes, ChronoUnit.MINUTES);
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyyMMddHHmm");
        DateTimeFormatter dateTimeFormatter2 = DateTimeFormatter.ofPattern("yyyyMMddHH");
        String time1 = minus.format(dateTimeFormatter);
        System.out.println(time1);
        String mm = time1.substring(time1.length() - 2, time1.length());
        System.out.println(mm);
        if (0 < Integer.parseInt(mm) && Integer.parseInt(mm) >= 30) {
            t1 = time1.substring(0, time1.length() - 2) + "00";
        } else if (Integer.parseInt(mm) < 30) {
            LocalDateTime lastHour = LocalDateTime.parse(time1, dateTimeFormatter).plus(1, ChronoUnit.HOURS);
            t1 = lastHour.format(dateTimeFormatter2) + "00";
        } else {
            t1 = time1;
        }

        t2 = t1.substring(0, t1.length() - 2);
        System.out.println("t1->" + t1 + "...t2->" + t2);
        return new String[]{t1, t2};
    }

    //查询出最新有游客数据的时间点
    private String[] newMinusMinutesStr(LocalDateTime datetime) {
        String t1, t2;//t1 yyyyMMddHHmm  t2 yyyyMMddHH
        LocalDateTime minus = datetime.minus(125, ChronoUnit.MINUTES);
        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyyMMddHHmm");
        DateTimeFormatter dateTimeFormatter2 = DateTimeFormatter.ofPattern("yyyyMMddHH");
        String time1 = minus.format(dateTimeFormatter);
        System.out.println(time1);
        String mm = time1.substring(time1.length() - 2, time1.length());
        System.out.println(mm);

        Integer i = Integer.parseInt(mm) / 15;
        if (i == 0) {
            t1 = time1.substring(0, time1.length() - 2) + "00";
        } else {
            t1 = time1.substring(0, time1.length() - 2) + String.valueOf(15 * i);
        }
        t2 = t1.substring(0, t1.length() - 2);
        System.out.println("t1->" + t1 + "...t2->" + t2);
        return new String[]{t1, t2};
    }


    /**
     * @param vertX polygon coordinates latitudes
     * @param vertY polygon coordinates longitudes
     * @return true indicate inside of the polygon， false indicate outside of the polygon
     */
    public Integer pnpolyAlgorithm(List<Double> vertX, List<Double> vertY, String timeId) {
        List<PassengerFlow> passengerFlows = passengerFlowMapper.selectByTime(timeId);
        int people = 0;
        for (PassengerFlow passengerFlow : passengerFlows) {
            double latitude = Double.parseDouble(passengerFlow.getLatitude());
            double longitude = Double.parseDouble(passengerFlow.getLongitude());
            if (CollectionUtils.isEmpty(vertX) || CollectionUtils.isEmpty(vertY)) {
                return 0;
            }
            double maxX = vertX.stream().max(Comparator.comparingDouble(Double::doubleValue)).get();
            double maxY = vertY.stream().max(Comparator.comparingDouble(Double::doubleValue)).get();
            double minX = vertX.stream().min(Comparator.comparingDouble(Double::doubleValue)).get();
            double minY = vertY.stream().min(Comparator.comparingDouble(Double::doubleValue)).get();

            /*if (longitude < minX || longitude > maxX || latitude < minY || latitude > maxY) {
                continue;
            }*/
            int i, j;
            boolean result = false;
            int n = vertX.size();
            Double[] vertx = vertX.toArray(new Double[0]);
            Double[] verty = vertY.toArray(new Double[0]);
            for (i = 0, j = n - 1; i < n; j = i++) {
                if ((verty[i] > latitude) != (verty[j] > latitude) &&
                        (longitude < (vertx[j] - vertx[i]) * (latitude - verty[i]) / (verty[j] - verty[i]) + vertx[i])) {
                    result = !result;
                }
            }
            if (result) {
                Integer allNums = Integer.parseInt(passengerFlow.getAllNums());
                people += allNums;
            }
        }
        return people;
    }
}
