package com.hisense.monitormanage.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.hisense.monitormanage.dto.RoadDataDtos;
import com.hisense.monitormanage.entity.RoadData;
import com.hisense.monitormanage.mapper.RoadDataMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author admin
 * @version 1.0.0
 * @ClassName RoadDataService.java
 * @Description 道路统计数据service，道路污染
 * @createTime 2022年05月11日 11:09:00
 */
@Service
@Slf4j
public class RoadDataService {
    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private RoadDataMapper roadDataMapper;

    //道路统计数据与排名，调用公开接口
    public List<Map> roadData() {
        String url = "http://outerdata.novaecs.com/api/qingdaoData/roadData?groupId={groupId}&timeType={timeType}&dt={dt}";

        Map<String, Object> map = new HashMap<>();

        map.put("dt", this.dateTimeStr());
        map.put("groupId", 1);
        map.put("timeType", "日");

        ResponseEntity<JSONObject> responseEntity;
        List<Map> list = new ArrayList<>();
        try {
            responseEntity = restTemplate.getForEntity(url, JSONObject.class, map);
            HttpStatus statusCode = responseEntity.getStatusCode();
            if (statusCode.is2xxSuccessful()) {
                JSONObject re = responseEntity.getBody();
                if (re.getIntValue("status") == 0) {
                    JSONArray jsonArray = re.getJSONArray("data");
                    list = JSONArray.parseArray(JSONObject.toJSONString(jsonArray), Map.class);
                }
            }
            return list;
        } catch (Exception e) {
            log.info("[roadData] exception:{}", e.getMessage());
            return list;
        }
    }

    //道路统计数据与排名(道路污染)，调用公开接口并保存到表t_road_data
    @Scheduled(cron = "0 0 7 * * ?")
    public void saveRoadData() {
        String url = "http://outerdata.novaecs.com/api/qingdaoData/roadData?groupId={groupId}&timeType={timeType}&dt={dt}";

        Map<String, Object> map = new HashMap<>();

        map.put("dt", this.dateTimeStr());
        map.put("groupId", 1);
        map.put("timeType", "日");

        ResponseEntity<JSONObject> responseEntity;
        List<Map> list = new ArrayList<>();
        try {
            responseEntity = restTemplate.getForEntity(url, JSONObject.class, map);
            HttpStatus statusCode = responseEntity.getStatusCode();
            if (statusCode.is2xxSuccessful()) {
                JSONObject re = responseEntity.getBody();
                if (re.getIntValue("status") == 0) {
                    JSONArray jsonArray = re.getJSONArray("data");
                    list = JSONArray.parseArray(JSONObject.toJSONString(jsonArray), Map.class);
                }
            }
        } catch (Exception e) {
            log.info("[roadData] exception:{}", e.getMessage());
        }
        if (list.size() > 0) {
            List<Map> maps = list.stream().map(l -> {
                l.put("points", JSONObject.toJSONString(l.get("points")));
                l.put("dt", this.dateTimeStr());
                return l;
            }).collect(Collectors.toList());
            List<List<Map>> lists = Lists.partition(maps, 200);
            lists.forEach(ll -> roadDataMapper.batchSave(ll));
        }
    }

    //根据日期查询道路污染数据
    public List<RoadData> listRoadData(String dt) {
        QueryWrapper<RoadData> wrapper = new QueryWrapper<>();
        wrapper.eq("dt", dt);
        List<RoadData> list = roadDataMapper.selectList(wrapper);
        return list;
    }

    //获取当前日期的时间串，2022-05-08 00:00:00
    private String dateTimeStr() {
        LocalDate localDate = LocalDate.now();
        LocalDate yestDay = localDate.minusDays(1);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime dateTime = LocalDateTime.of(yestDay.getYear(), yestDay.getMonth(), yestDay.getDayOfMonth(), 0, 0, 0);
        return dateTime.format(formatter);
    }

    //获取当前日期的时间串，2022-05-08
    private String dateStr() {
        LocalDate localDate = LocalDate.now();
        LocalDate yestDay = localDate.minusDays(1);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        return yestDay.format(formatter);
    }
}
