package com.hisense.monitormanage.service;

import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.incrementer.DefaultIdentifierGenerator;
import com.baomidou.mybatisplus.core.incrementer.IdentifierGenerator;
import com.google.common.collect.Lists;
import com.hisense.monitormanage.entity.CaseCityLaw;
import com.hisense.monitormanage.entity.Result;
import com.hisense.monitormanage.mapper.CaseCityLawMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author admin
 * @version 1.0.0
 * @ClassName SJZTService.java
 * @Description 数交中台service, 城市执法
 * @createTime 2022年05月12日 18:00:00
 */
@Service
@Slf4j
public class SJZTService {
    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private CaseCityLawMapper caseCityLawMapper;

    //数交中台
    //1.登录
    public String sjztToken() {
        String url = "http://120.221.95.13:9090/apin/authorization/oauth/token";
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", "Basic dGVzdF9jbGllbnQ6ZTk4OWQ0NmZkYmMxYzM3NmMxOWE0M2FhZjg1MjI3YTQ=");
        headers.add("Content-Type", "application/x-www-form-urlencoded");

        MultiValueMap map = new LinkedMultiValueMap();
        map.add("username", "qdsyn");
        String sigin = SecureUtil.md5("Qdsyn123!");
        map.add("password", sigin);//Qdsyn123!,需要MD5加密
        map.add("scope", "read");
        map.add("grant_type", "password");
        map.add("vc", "NO");

        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(map, headers);
        ResponseEntity<JSONObject> responseEntity;
        try {
            responseEntity = restTemplate.exchange(url, HttpMethod.POST, request, JSONObject.class);
            JSONObject jsonObject = responseEntity.getBody();
            if (jsonObject.getBooleanValue("success") == true) {
                JSONObject resultJson = jsonObject.getJSONObject("result");
                return resultJson.getString("accessToken");
            } else {
                return null;
            }
        } catch (Exception e) {
            log.info("[qidiToken] exception:{}", e.getMessage());
            return null;
        }
    }

    //从接口获取案件数据
    public List<Map> listSJZTDatas() {
        String token = this.sjztToken();
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", "Bearer " + token);

        Map<String, Object> map = new HashMap<>();
        JSONObject search = new JSONObject();
        search.put("opt", "LIKE");
        search.put("key", "WTLYMC");
        search.put("val", "微信举报");

        map.put("json", search);
        int i = 1;
        int roundCount = 0;
        int count = 0;
        List<Map> maps = new ArrayList<>();
        do {
            String url = "http://120.221.95.13:9090/catalog/resCatalogApply/getData/UC_QUERY_QDZWT_307013311122000064_1?pageSize=10000&pageNo={1}&search=[{2}]";
            HttpEntity<Map> httpEntity = new HttpEntity<>(null, headers);
            ResponseEntity<JSONObject> responseEntity;
            try {
                responseEntity = restTemplate.exchange(url, HttpMethod.GET, httpEntity, JSONObject.class, i, search);

                JSONObject jsonObject = responseEntity.getBody();
                count = jsonObject.getIntValue("count");
                JSONArray jsonArray = jsonObject.getJSONArray("list");
                maps.addAll(JSONObject.parseArray(JSONObject.toJSONString(jsonArray), Map.class));
                i++;
                roundCount = i * 10000;
            } catch (Exception e) {
                log.info("[listSJZTDatas] exception:{}", e.getMessage());
                return null;
            }
        } while (roundCount <= count);

        log.info("[listSJZTDatas] 返回数据的数量:{}", maps.size());
        return maps;
    }

    //从接口获取案件数据并保存
    public void batchSave() {
        String token = this.sjztToken();
        HttpHeaders headers = new HttpHeaders();
        headers.add("Authorization", "Bearer " + token);

        Map<String, Object> map = new HashMap<>();
        JSONObject search = new JSONObject();
        search.put("opt", "LIKE");
        search.put("key", "AJJLSJ");
//        search.put("val",this.dateStr());
        search.put("val", "2017");

        map.put("json", search);
        int i = 1;
        int roundCount = 0;
        int count = 0;
        List<Map> mapList = new ArrayList<>();
        do {
            String url = "http://120.221.95.13:9090/catalog/resCatalogApply/getData/UC_QUERY_QDZWT_307013311122000064_1?pageSize=10000&pageNo={1}&search=[{2}]";
            HttpEntity<Map> httpEntity = new HttpEntity<>(null, headers);
            ResponseEntity<JSONObject> responseEntity;
            try {
                responseEntity = restTemplate.exchange(url, HttpMethod.GET, httpEntity, JSONObject.class, i, search);

                JSONObject jsonObject = responseEntity.getBody();
                count = jsonObject.getIntValue("count");
                JSONArray jsonArray = jsonObject.getJSONArray("list");
                mapList.addAll(JSONObject.parseArray(JSONObject.toJSONString(jsonArray), Map.class));
                i++;
                roundCount = i * 10000;
            } catch (Exception e) {
                log.info("[SJZTService-batchSave] exception:{}", e.getMessage());
            }
        } while (roundCount <= count);
        System.out.println(mapList.size());

        IdentifierGenerator identifierGenerator = new DefaultIdentifierGenerator();
        mapList.forEach(m -> m.put("id", identifierGenerator.nextId(new Object())));//给id赋值

        List<List<Map>> lists = Lists.partition(mapList, 50);
        try {
            lists.forEach(l -> caseCityLawMapper.batchSave(l));
        } catch (Exception e) {
            e.printStackTrace();
            log.error("[SJZTService batchsave Exception]:", e.getMessage());
        }

    }

    public List<CaseCityLaw> listSJZTByDt(String ajjlsj) {
        QueryWrapper<CaseCityLaw> wrapper = new QueryWrapper<>();
        wrapper.like("ajjlsj", ajjlsj);
        return caseCityLawMapper.selectList(wrapper);
    }

    public Result upStream(JSONObject data) {
        String url = "?actionType={1}&senderCode={2}&data={3}";
        String senderCode = "";//发送方标识
        String actionType = "UP_REC_REPORT";//调用方法，固定值
        boolean result = false;

        ResponseEntity<JSONObject> responseEntity;
        try {
            responseEntity = restTemplate.postForEntity(url, null, JSONObject.class, senderCode, actionType, data);
            JSONObject responseJSON = responseEntity.getBody();
            if (responseEntity.getStatusCodeValue() == 200) {
                if (responseJSON.getBooleanValue("success") && responseJSON.getIntValue("code") == 0) {
                    return Result.success(responseJSON.get("data"));
                } else {
                    return Result.error(responseJSON.getString("data"));
                }
            } else {
                return Result.error(String.valueOf(responseEntity.getStatusCodeValue()));
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("[SJZTService upStream] Exception:" + e.getMessage());
            return Result.error(e.getMessage());
        }
    }

    //获取当前日期的时间串，2022-05-08 00:00:00
    private String dateTimeStr() {
        LocalDate localDate = LocalDate.now();
        LocalDate yestDay = localDate.minusDays(1);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime dateTime = LocalDateTime.of(yestDay.getYear(), yestDay.getMonth(), yestDay.getDayOfMonth(), 0, 0, 0);
        return dateTime.format(formatter);
    }

    //获取当前日期的时间串，2022-05-08
    private String dateStr() {
        LocalDate localDate = LocalDate.now();
        LocalDate yestDay = localDate.minusDays(1);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        return yestDay.format(formatter);
    }
}
