package com.hisense.monitormanage.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.hisense.monitormanage.entity.Result;
import com.hisense.monitormanage.entity.SanitationTask;
import com.hisense.monitormanage.mapper.SanitationMapper;
import com.hisense.monitormanage.mapper.SanitationTaskMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author admin
 * @version 1.0.0
 * @ClassName SanitationService.java
 * @Description TODO
 * @createTime 2022年05月19日 11:23:00
 */
@Service
@Slf4j
public class SanitationService {
    @Autowired
    private SanitationMapper sanitationMapper;
    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private QidiService qidiService;
    @Autowired
    private SanitationTaskMapper sanitationTaskMapper;

    public Result saveSanitationRoad() {
        List<Map> list = qidiService.resCatalogApplyHJWSRoad();
        try {
            if (list.size() > 0) {
                List<List<Map>> maps = Lists.partition(list, 100);
                maps.forEach(m -> sanitationMapper.batchSaveSanitationRoad(m));
            }
            return Result.success();
        } catch (Exception e) {
            e.printStackTrace();
            log.error("[SanitationService saveSanitationRoad] Exception:" + e.getMessage());
            return Result.error(e.getMessage());
        }
    }

    public Result saveSanitationTask() {
        List<Map> list = qidiService.resCatalogApplyHJWSZY();
        try {
            if (list.size() > 0) {
                List<List<Map>> maps = Lists.partition(list, 100);
                maps.forEach(m -> sanitationMapper.batchSaveSanitationTask(m));
            }
            return Result.success();
        } catch (Exception e) {
            e.printStackTrace();
            log.error("[SanitationService saveSanitationTask] Exception:" + e.getMessage());
            return Result.error(e.getMessage());
        }
    }

    //批量查询环卫车作业数据并保存，可以根据实际需求更改
    @Scheduled(cron = "0 0 8 * * ?")
    public void saveSanitationTaskScheduled() {
        QueryWrapper<SanitationTask> wrapper = new QueryWrapper<>();
        wrapper.orderByDesc("updatetime").last("limit 1");
        SanitationTask sanitationTask = sanitationTaskMapper.selectOne(wrapper);
        List<Map> maps = new ArrayList<>();

        try {
            if (sanitationTask != null) {
                maps = qidiService.resCatalogApplyHJWSZY(String.valueOf(sanitationTask.getUpdatetime()));

            } else {
                maps = qidiService.resCatalogApplyHJWSZY();
            }

            if (maps.size() > 0) {
                List<List<Map>> lists = Lists.partition(maps, 100);
                lists.forEach(list -> sanitationMapper.batchSaveSanitationTask(list));
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("[SanitationService saveSanitationTaskScheduled] Exception:" + e.getMessage());
        }
    }

}
