package com.hisense.monitormanage.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.hisense.monitormanage.entity.SedimentTrail;
import com.hisense.monitormanage.mapper.SedimentTrailMapper;
import com.hisense.monitormanage.utils.LongLatUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author admin
 * @version 1.0.0
 * @ClassName SedimentTrailService.java
 * @Description 渣土车service
 * @createTime 2022年05月09日 14:01:00
 */
@Service
@Slf4j
public class SedimentTrailService {
    @Autowired
    private SedimentTrailMapper sedimentTrailMapper;

    @Autowired
    private QidiService qidiService;

    //获取渣土车轨迹并保存到表t_trail_sediment
    //@Transactional(rollbackFor = Exception.class)
    public boolean batchSaveSedimentTrail() {
        try {
            List<Map> maps = qidiService.resCatalogApplyZTYS();
            Lists.partition(maps, 200).forEach(
                    item -> sedimentTrailMapper.batchaSave(item)
            );
            return true;
        } catch (Exception e) {
            log.error("[batchSaveSedimentTrail] exception:{}", e.getMessage());
            return false;
        }
    }

    /**
     * 根据输入的经纬度和查询的半径（米）查询范围内的渣土车轨迹信息
     *
     * @param longitude（经度） latitude（纬度） radius(米) start(时间) end(时间)
     */
    public List<SedimentTrail> listSedimentTrailByPoints(double longitude, double latitude, Integer radius, String start, String end) {
        double[] points = LongLatUtil.getAround(longitude, latitude, radius);
        QueryWrapper<SedimentTrail> wrapper = new QueryWrapper<>();
        wrapper.between("update_time", start, end).between("longitude", points[0], points[2]).between("latitude", points[1], points[3])
                .orderByAsc("sim_card_num", "record_time");
        List<SedimentTrail> list = sedimentTrailMapper.selectList(wrapper);
        return list;
    }
}
