package com.hisense.monitormanage.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.hisense.monitormanage.entity.TrailSanitation;
import com.hisense.monitormanage.mapper.TrailSanitationMapper;
import com.hisense.monitormanage.utils.LongLatUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

@Service
@Log4j2
public class TrailSanitationService {
    @Autowired
    private TrailSanitationMapper trailSanitationMapper;
    @Autowired
    private QidiService qidiService;

    public boolean saveTrailSanitation() {
        boolean result = false;
        try {
            List<Map> list = qidiService.resCatalogApplyHJWS();
            List<List<Map>> lists = Lists.partition(list, 200);
            lists.forEach(l -> trailSanitationMapper.batchaSave(l));
            result = true;
        } catch (Exception e) {
            log.error("[saveTrailSanitation] Exception:", e.getMessage());
        }
        return result;
    }

    /**
     * 根据输入的经纬度和查询的半径（米）查询范围内的环卫车轨迹信息
     *
     * @param longitude（经度） latitude（纬度） raidus(米) start(时间) end(时间)
     */
    public List<TrailSanitation> listSedimentTrailByPoints(double longitude, double latitude, Integer radius, String start, String end) {
        double[] points = LongLatUtil.getAround(longitude, latitude, radius);
        QueryWrapper<TrailSanitation> wrapper = new QueryWrapper<>();
        wrapper.between("fssj", start, end).between("jd", points[0], points[2]).between("wd", points[1], points[3]);
        return trailSanitationMapper.selectList(wrapper);
    }

}
